/***************************************************************************
                                kcmdline.cpp
                             -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kcmdline.h"
#include <kprocess.h>
#include <qiconset.h>
#include <qwhatsthis.h>
#include <unistd.h>
#include "kicons.h"
#include "krusader.h"
#include "resources.h"
#include "defaults.h"
#include <qdir.h>

KCMDLine::KCMDLine(QWidget *parent, const char *name ) : QWidget(parent,name) {
  // config
  krConfig->setGroup("Look&Feel");
  /////////////////////////////////////////////
  QGridLayout *layout=new QGridLayout(this,1,4);
  path=new QLabel(this);
  path->setAlignment(Qt::AlignRight);
  path->setFrameStyle(QFrame::Box | QFrame::Sunken);
  path->setLineWidth(1);
  path->setFont(krConfig->readFontEntry("Cmdline Font",_CmdlineFont));
  layout->addWidget(path,0,0);
  // and editable command line
  cmdLine=new KRLineEdit(this);
  cmdLine->setFont(krConfig->readFontEntry("Cmdline Font",_CmdlineFont));
  QWhatsThis::add(cmdLine,WT_CMDLINE_EDIT);
  layout->addWidget(cmdLine,0,1);
  // history button
  history=new QToolButton(this);
  history->setTextLabel("Show the last commands you typed in the command line.");
  history->setIconSet(QIconSet(history_xpm));
	historyMenu=new QPopupMenu(this);
	historyMenu->setFont(krConfig->readFontEntry("History Font",_HistoryFont));
  history->setPopup(historyMenu);	
  history->setPopupDelay(100);
	connect(historyMenu,SIGNAL(activated(int)),this,
          SLOT(setCmdLineText(int)));	
	QWhatsThis::add(history,WT_CMDLINE_HISTORY);
  layout->addWidget(history,0,2);
  // a run in terminal button
  terminal=new QToolButton(this);
  terminal->setTextLabel("If pressed, Krusader executes command line in a terminal.");
  terminal->setToggleButton(true);
  terminal->setOn(false);
  terminal->setOffIconSet(QIconSet(terminal32x32_xpm));
  QWhatsThis::add(terminal,WT_CMDLINE_TERMINAL);
  layout->addWidget(terminal,0,3);
  // connections
  connect(cmdLine,SIGNAL(returnPressed(const QString&)),this,
          SLOT(slotRun(const QString&)));
  connect(cmdLine,SIGNAL(upkeyPressed()),this,
  				SLOT(slotHistoryUp()));
  connect(cmdLine,SIGNAL(downkeyPressed()),this,
  				SLOT(slotHistoryDown()));
	historySize=0;
	currentItem=-1;	// which means up key wasn't pressed yet
	historyList.setAutoDelete(true);
}

KCMDLine::~KCMDLine(){
}

void KCMDLine::slotRun(const QString& command){
  krConfig->setGroup("External"); 	
  ///////////////////////////////
 	cmdLine->clear();
	addToHistory(command);
	currentItem=-1;	// reset the up/down keys
  QString panelPath=path->text().left( path->text().length()-1);
  	
	if (command.left(3)=="cd ") { // cd command effect the active panel
		QString dir = command.right(command.length()-command.find(" ")).stripWhiteSpace();
		if (dir == "~") dir = QDir::homeDirPath();
		else
		if (dir.left(1)!="/") dir=panelPath+(panelPath=="/" ? "" : "/")+dir;
		krApp->mainView->activePanel->slotPanelRefresh(dir);
	} else {
		QString save = getcwd(0,0);
		KShellProcess proc;
		chdir( panelPath );
		// run in a terminal ???
		if (terminal->isOn()) proc << krConfig->readEntry("Terminal",_Terminal)
		                           << krConfig->readEntry("Terminal Exec",_TerminalExec);
		proc <<  command;
  	proc.start(KProcess::DontCare);
		chdir(save);
	}
}

void KCMDLine::slotHistoryUp() {
	if (currentItem==((signed)historyList.count())-1) return;	// top of the history list
	cmdLine->setText(*historyList.at(++currentItem));
}

void KCMDLine::slotHistoryDown() {
	if (currentItem<=0) return;	// top of the history list
	cmdLine->setText(*historyList.at(--currentItem));
}
	
// history remembers up to 20 commands
//////////////////////////////////////
void KCMDLine::addToHistory(QString cmd) {
	if (historyList.count()==20)
		if (!historyList.removeLast()) {
			KDEBUG("KCmdLine: Unable to remove item from list. bailing out");
			exit(0);
		}
	QString *temp=new QString(cmd);
	if (!historyList.insert(0,temp)) {
		KDEBUG("KCmdLine: Unable to allocate memory. bailing out");
		exit(0);
	}
	refreshHistory();
}

void KCMDLine::refreshHistory() {
  historyMenu->clear();   // just in case
	for (int i=historyList.count()-1; i>=0; --i)	
	    historyMenu->insertItem(*historyList.at(i),i);
}

void KCMDLine::setCmdLineText(int id) {
	cmdLine->setText(*historyList.at(id));
	currentItem=id;
}
