/***************************************************************************
                                kmountman.cpp
                             -------------------
    version              : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "kmountman.h"
// QT incldues
#include <qcstring.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qmessagebox.h>
// KDE includes
#include <kprocess.h>
#include <ktempfile.h>
#include <kio/job.h>
#include <klocale.h>
// Krusader includes
#include "resources.h"
#include "krusader.h"
#include <fstream>
#include <qfile.h>
#include <string>

KMountMan::KMountMan() : QObject(),Ready(false),
                         Operational(false) {
	filesystems.setAutoDelete(true);	
	localDf=new fsData();				 // will be used to move around information
  forceUpdate();
}

KMountMan::~KMountMan(){
}

QString KMountMan::followLink(QString source) {
  QFileInfo temp(source);
  while (temp.isSymLink()) temp.setFile(temp.readLink());
  return temp.fileName();
}

bool KMountMan::createFilesystems() {
  string temp[3][100];  // a temporary array which allows parsing of upto 100 fs
  string dumb;           // my very-own dumb pipe: nothing goes out of it.
  int i=0,j=0;

  noOfFilesystems=0;
  // if |filesystems|>0, than we are re-creating, delete the old list
  if (filesystems.count()>0) while (filesystems.removeLast());
  // open the /etc/fstab file...
  ifstream fstab("/etc/fstab");
  if (!fstab.is_open()) {
    KDEBUG("Mt.Man: Unable to read /etc/fstab !!! Shutting down. (sorry)");
    return false;
  }
  // and read it into the temporary array
  while (!fstab.eof()) {
    // temp[0]==name, temp[1]==type, temp[2]==mount point
    fstab >> (temp[0][i]) >> (temp[2][i]) >> temp[1][i] >> dumb >> dumb >> dumb;
    ++i;
  }
  fstab.close();  // finished with it
  for (j=0; j<=i; ++j) {
  	if (temp[0][j]=="" || temp[1][j]=="proc" || temp[1][j]=="/dev/pts" ||
  			temp[1][j]=="swap" || temp[0][j]=="none") continue;
  	++noOfFilesystems;
  }
  // now, create the main list
//  filesystems=new fsData[noOfFilesystems];
  QString forDebugOnly="";
  i=0; j=0;
  while (i<noOfFilesystems) {
		if (temp[0][j]=="" || temp[1][j]=="proc" || temp[1][j]=="/dev/pts" ||
  			temp[1][j]=="swap" || temp[0][j]=="none") ++j;
    else {
	    fsData* system=new fsData();
	    system->setName(temp[0][j].c_str());
  	  system->setType(temp[1][j].c_str());
   	  system->setMntPoint(temp[2][j].c_str());
  	  filesystems.append(system);
  	  forDebugOnly=forDebugOnly+"["+system->name()+"] ";
  	  ++i;++j;
    }
  }
	KDEBUG("Mt.Man: "+forDebugOnly+" found. Trying DF...");
	return true;
}

// run DF process and when it finishes, catch output with "parseDfData"
///////////////////////////////////////////////////////////////////////
void KMountMan::updateFilesystems() {
  // create the "df -P -T" process
  KShellProcess dfProc;
  KTempFile *temp=new KTempFile();
  temp->setAutoDelete(true);

  dfProc << "df" << "-T" << "-P" << ">" << temp->name();
  dfProc.start(KProcess::Block);  // wait for DF to finish
  // call parseDfData to work on the data
  parseDfData(temp->name());
  delete temp;
}

// service functions
QString KMountMan::nextWord(QString &s,char c) {
  s=s.stripWhiteSpace();
  int j=s.find(c,0);
  QString temp=s.left(j); // find the leftmost word.
  s.remove(0,j);
  return temp;
}

fsData* KMountMan::location(QString name) {
  fsData* it;
  for (it=filesystems.first() ; (it!=0) ; it=filesystems.next())
   if (followLink(it->name())==followLink(name)) break;
  return it;
}

// parseDfData assumes that DF always knows better than FSTAB about currently
// mounted filesystems.
/////////////////////////////////////////////////////////////////////////////
void KMountMan::parseDfData(QString filename) {
  QString temp;
  QFile f(filename);
  if (!f.open(IO_ReadOnly)) { // error reading temp file
    Operational=false;
    Ready=false;
    return;                   // make mt.man non-operational
  }
  QTextStream t( &f );        // use a text stream
  QString s;
  s = t.readLine();  // read the 1st line - it's trash for us
  // now that we have a QString containing all the output of DF, let's get to work.
  int countFilesystems=0; // sucessfully found filesystems
  while (!t.eof()) {
    s = t.readLine();  // this is the important one!
    temp=nextWord(s,' ');
    temp=followLink(temp);  // make sure DF gives us the true device and not a link
    fsData* loc=location(temp); // where is the filesystem located in our list?
    if (loc==0) {
      KDEBUG("Mt.Man: filesystem ["+temp+"] found by DF is unlisted in /etc/fstab.");
      loc=new fsData();
      filesystems.append(loc);
    }
    temp=nextWord(s,' ');   // catch the TYPE
    loc->setType(temp);
    if (loc->type()!=temp) {
    	KDEBUG("Mt.Man: according to DF, filesystem ["+loc->name()+
    				 "] has a different from what's stated in /etc/fstab.");
      loc->setType(temp);  // DF knows best
    }
    temp=nextWord(s,' ');
    loc->setTotalBlks(temp.toLong());
    temp=nextWord(s,' ');
    loc->setUsedBlks(temp.toLong());
    temp=nextWord(s,' ');   // get rid of the next 2 words
    temp=nextWord(s,' ');
    temp=nextWord(s,'\n');  // read the "mounted on" thing
    if (loc->mntPoint()!=temp) {
    	KDEBUG("Mt.Man: according to DF, filesystem ["+loc->name()+
    				 "] is mounted on a point which is different from what's stated in /etc/fstab.");
      loc->setMntPoint(temp);  // DF knows best
    }
    loc->setMounted(true);
    ++countFilesystems;
  }
  Operational=Ready=true; // we are finished here
  if (countFilesystems>0) KDEBUG("Mt.Man: Alive and kicking. ");
   else {
    KDEBUG("Mt.Man: failed running DF. Shutting down (sorry).");
    Operational=Ready=false;
   }
}

// works the same as parseDfData, only for single files
///////////////////////////////////////////////////////
void KMountMan::parseDf(QString filename) {
  QFile f(filename);
  if (!f.open(IO_ReadOnly)){  // error reading temp file
    Operational=false;
    Ready=false;
    return;                   // make mt.man non-operational
  }
  QTextStream t( &f );        // use a text stream
  QString s;
  s = t.readLine();  // read the 1st line - it's trash for us
  s = t.readLine();  // this is the important one!
  localDf->setName(nextWord(s,' '));
  localDf->setType(nextWord(s,' '));
  localDf->setTotalBlks( nextWord(s,' ').toLong() );
  localDf->setUsedBlks( nextWord(s,' ').toLong() );
  nextWord(s,' ');  nextWord(s,' ');
	localDf->setMntPoint(nextWord(s,'\n'));
  f.close();
  Ready=true;
}

// return the information about where does PATH resides
///////////////////////////////////////////////////////
fsData KMountMan::getData(QString path) {
  KShellProcess dfProc;
  KTempFile *temp=new KTempFile();
  temp->setAutoDelete(true);

  dfProc << "df" << "-T" << "-P" << "\""+path+"\"" << ">" << temp->name();
  // lower the ready flag, wait for DF to finish
  Ready=false;
  dfProc.start(KProcess::Block);
  parseDf(temp->name());
  KDEBUG("Mt.Man: finished info request for "+path+".");
  delete temp;
  return *localDf;
}

void KMountMan::mountResult(KIO::Job* job) {
  if (job->error()) job->showErrorDialog();
  forceUpdate();
}

void KMountMan::forceUpdate() {
	KDEBUG("Mt.Man: Born, looking around to get familiar.");
  // first create our list from /etc/fstab
  if (!createFilesystems()) {  // create the potential fs from /etc/fstab
  	Operational=Ready=false;
		return;										 // if something went wrong, bail out!
	}
	updateFilesystems();         // use the output of "DF -T -P" to update data
  emit updated();
}

void KMountMan::mount(fsData *p) {
	bool ro=(p->type()=="iso9660" || p->name()=="/dev/cdrom");
	KIO::SimpleJob* job=KIO::mount(ro,p->type(),p->name(),p->mntPoint(),true);
	connect( job, SIGNAL( result( KIO::Job * ) ),
    this, SLOT( mountResult( KIO::Job * ) ) );
}

void KMountMan::unmount(fsData *p) {
	KIO::SimpleJob* job=KIO::unmount(p->mntPoint(),false);
	connect( job, SIGNAL( result( KIO::Job * ) ),
    this, SLOT( mountResult( KIO::Job * ) ) );
}

void KMountMan::toggleMount(QString device) {
  fsData* p=location(device);
  if (!p)  { // something very very wierd happened
    QMessageBox::critical(0, i18n("ERROR"), i18n("MountMan has an internal error.\nPlease notify the developers.\nThank you.")
    , i18n("Exit"));
    exit(0);
  }
  // request mountMan to (un)mount something, if he does his job (and he does)
  // we will be notified by signal
  (p->mounted() ? unmount(p) : mount(p));
}

void KMountMan::autoMount(QString path) {
  krConfig->setGroup("Advanced");
  if (!krConfig->readBoolEntry("AutoMount",_AutoMount)) return;
  fsData* it;
  for (it=filesystems.first() ; (it!=0) ; it=filesystems.next())
   if (it->mntPoint()==path) break;
  if (!it) return;  // if not a mount-point, do nothing
  if (!it->mounted()) toggleMount(it->name());
}
