/***************************************************************************
                                kmountmangui.cpp
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "kmountmangui.h"
#include "krusader.h"
#include "kicons.h"
#include <klocale.h>
#include <qpixmap.h>
#include <kpopupmenu.h>
#include <qbitmap.h>
#include <qmessagebox.h>

KMountManGUI::KMountManGUI() : KDialogBase(0,0,true,"Mount.Man"),
      mountList(0){
  connect(&krMtMan,SIGNAL(updated()),this,SLOT(updateList()));
  connect(&watcher,SIGNAL(dirty()),&krMtMan,SLOT(forceUpdate()));
  setButtonOKText(i18n("Close"),i18n("Close the Mount.Man window."),0);
  showButtonApply(false); showButtonCancel(false);
  setPlainCaption(i18n("MountMan - Your Mount-Manager"));
  widget=new KJanusWidget(this,0,KJanusWidget::Tabbed);
  createLayout();
  setMainWidget(widget);
  widget->setMinimumSize(widget->sizeHint().width(),widget->sizeHint().height());
  setMinimumSize(widget->minimumSize());
  setMaximumSize(widget->minimumSize());  // unresizeable
  // connections
  connect(mountList,SIGNAL(doubleClicked(QListViewItem *)),this,
          SLOT(doubleClicked(QListViewItem*)));
  connect(mountList,SIGNAL(clicked(QListViewItem *)),this,
          SLOT(clicked(QListViewItem*)));
  exec();
}

void KMountManGUI::createLayout() {
  mainPage=widget->addPage(i18n("Filesystems"),0);
  createMainPage();
  widget->showPage(Filesystems);
}

void KMountManGUI::createMainPage() {
  // check if we need to clean up first!
  if (mountList!=0) {
    mountList->hide();
    delete mountList;
    mountList=0;
  }
  // clean up is finished...
  QGridLayout *layout=new QGridLayout(mainPage,1,1);
  mountList=new QListView(mainPage);  // create the main container
  krConfig->setGroup("Look&Feel");
  mountList->setFont(krConfig->readFontEntry("Filelist Font",_FilelistFont));
	mountList->setAllColumnsShowFocus(true);
	mountList->setMultiSelection(false);
	mountList->setVScrollBarMode(QScrollView::AlwaysOn);
	mountList->setHScrollBarMode(QScrollView::Auto);
	mountList->setShowSortIndicator(true);
	int i=QFontMetrics(mountList->font()).width("W");
	int j=QFontMetrics(mountList->font()).width("0");
	j=(i>j ? i : j);
	mountList->addColumn(i18n("Name"),j*8);
	mountList->addColumn(i18n("Type"),j*4);
	mountList->addColumn(i18n("Mnt.Point"),j*6);
	mountList->addColumn(i18n("Total Size"),j*6);
	mountList->addColumn(i18n("Free Size"),j*6);
	mountList->addColumn(i18n("Free %"),j*5);
	mountList->addColumn(i18n("Mnt'd"),j*4);
	mountList->setColumnWidthMode(0,QListView::Maximum);
	mountList->setColumnWidthMode(1,QListView::Maximum);
	mountList->setColumnWidthMode(2,QListView::Maximum);
	mountList->setColumnWidthMode(3,QListView::Maximum);
	mountList->setColumnWidthMode(4,QListView::Maximum);
	mountList->setColumnWidthMode(5,QListView::Maximum);
  mountList->setColumnAlignment(6,Qt::AlignCenter);
  // now the list is created, time to fill it with data.
  krMtMan.forceUpdate();
  layout->addWidget(mountList,0,0);
}

void KMountManGUI::updateList() {
  createDirwatch();
  fsData *it;
  mountList->clear();
  for (it=krMtMan.filesystems.first(); it!=0 ; it=krMtMan.filesystems.next()) {
    bool mtd=it->mounted();
    QString tSize=QString("%1").arg(KIO::convertSize(it->totalBytes()));
    QString fSize=QString("%1").arg(KIO::convertSize(it->freeBytes()));
    QString sPrct=QString("%1%").arg(100-(it->usedPerct()));
    QListViewItem *item=new QListViewItem(mountList,it->name(),
       it->type(),it->mntPoint(),
       (mtd ? tSize : QString("N/A")),(mtd ? fSize : QString("N/A")),
       (mtd ? sPrct : QString("N/A")));
    item->setPixmap(6,(mtd ? yes_xpm : no_xpm));
 		QString id=krMtMan.followLink(it->name()).left(2);
 		QPixmap *icon;
 		if (id=="fd") icon=new QPixmap(floppy_xpm);
 		  else
 		if (id=="hd") icon=new QPixmap(harddisk_xpm);
 		  else
 		if (id=="cd" || it->type()=="iso9660") icon=new QPixmap(cdrom_xpm);
 		  else
 		icon=new QPixmap(harddisk_xpm);    // our default
 		
    item->setPixmap(0,*icon);
    delete icon;
  }
  watcher.startScan();
}

void KMountManGUI::createDirwatch() {
  fsData* it;
  for (it=krMtMan.filesystems.first(); it!=0 ; it=krMtMan.filesystems.next()) {
    watcher.addDir(it->mntPoint());
  }
}

void KMountManGUI::doubleClicked(QListViewItem *i) {
  if (!i) return;
  krMtMan.toggleMount(i->text(0));
}

void KMountManGUI::clicked(QListViewItem *item) {
  // these are the values that will exist in the menu
  #define MOUNT_ID       90
  #define UNMOUNT_ID     91
  #define FORMAT_ID      93
  #define PROPERTIES_ID  99
  //////////////////////////////////////////////////////////
  if (!item) return;
  fsData *system=krMtMan.location(item->text(0));
  if (!system) {
    QMessageBox::critical(0, i18n("ERROR"), i18n("MountMan has an internal error.\nPlease notify the developers.\nThank you.")
    , i18n("Exit"));
    exit(0);
  }
  // create the menu
  KPopupMenu popup;
  popup.insertTitle(i18n("MountMan"));
  popup.insertItem(i18n("Mount"),MOUNT_ID);
  popup.insertItem(i18n("Unmount"),UNMOUNT_ID);
  popup.insertSeparator();
  popup.insertItem(i18n("Format"),FORMAT_ID);
  popup.insertSeparator();
  popup.insertItem(i18n("Properties"),PROPERTIES_ID);
  // ENABLE/DISABLE items accordingly
  popup.setItemEnabled(PROPERTIES_ID,false);
  popup.setItemEnabled(FORMAT_ID,false);
  popup.setItemEnabled((system->mounted() ? MOUNT_ID : UNMOUNT_ID),false);
  int result=popup.exec(QCursor::pos());
  // check out the user's option
  switch (result) {
    case -1 : return;     // the user clicked outside of the menu
    case MOUNT_ID   :
    case UNMOUNT_ID :
      krMtMan.toggleMount(system->name());
      break;
    case FORMAT_ID :
      break;
    case PROPERTIES_ID :
      break;
  }
}
