/***************************************************************************
                                konfigurator.cpp
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "konfigurator.h"
#include "krusader.h"
#include <kfiledialog.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <klocale.h>
#include <qwhatsthis.h>
#include <qfontmetrics.h>
#include <kcolorbtn.h>
#include "kicons.h"
#include <iostream>

#define MAXWIDTH 200

Konfigurator::Konfigurator() : KDialogBase(0,0,true,"Konfigurator") {
  setButtonOKText(i18n("Accept"),i18n("GUI related choices and startup options will be done in next activation of Krusader"),0);
  // we don't use apply, instead, RESET TO DEFAULTS
  setButtonApplyText(i18n("Defaults"),i18n("Reset ALL of Krusader's options to factory defaults"),0);
  setButtonCancelText(i18n("Discard"),i18n("Dump all the changes you made!"),0);
  setButtonWhatsThis(Ok,i18n("Accept the changes you made. GUI (and startup) related changes will be updated the next time you run krusader"));
  setButtonWhatsThis(Apply,i18n("Return the settings to Krusader's defaults."));
  setButtonWhatsThis(Cancel,i18n("Discard all changes."));
  setPlainCaption(i18n("Konfigurator - Creating Your Own Krusader"));
  krConfig->setGroup("Advanced");
  bool tabInterface=krConfig->readBoolEntry("Tab Interface",_TabInterface);
  if (tabInterface)
    widget=new KJanusWidget(this,0,KJanusWidget::Tabbed);
    else {
      widget=new KJanusWidget(this,0,KJanusWidget::TreeList);
      widget->setTreeListAutoResize(true);
      widget->setShowIconsInTreeList(true);
      widget->setRootIsDecorated(true);
    }
  createLayout();
  setMainWidget(widget);
  if (tabInterface) {
    widget->setMinimumSize(widget->sizeHint().width(),widget->sizeHint().height());
    setMinimumSize(widget->minimumSize());
    setMaximumSize(widget->minimumSize());  // unresizeable
  }
  exec();
}


Konfigurator::~Konfigurator(){
}

QString fontType(QFont *font) {
  QString t1;
  t1.sprintf("%d",font->pointSize());
  QString t=font->family()+", "+t1;
  return t;
}

void Konfigurator::slotOk(){
  // write settings for the General group
  krConfig->setGroup("General");
	krConfig->writeEntry("Mark Dirs",G_markDirs->isChecked());
	krConfig->writeEntry("Mimetype Magic",G_useMagic->isChecked());
	krConfig->writeEntry("Show Hidden",G_showHidden->isChecked());
	krConfig->writeEntry("Remember Position",G_rememberPos->isChecked());
	krConfig->writeEntry("Left Starts Tree",G_leftTree->isChecked());
	krConfig->writeEntry("Right Starts Tree",G_rightTree->isChecked());	
  krConfig->writeEntry("Show Splashscreen",G_splashScreen->isChecked());
	krConfig->writeEntry("Left Origin",G_leftOrigin->text());
	krConfig->writeEntry("Right Origin",G_rightOrigin->text());
	// write settings for the LookNFeel group
  krConfig->setGroup("Look&Feel");
	krConfig->writeEntry("Origin Background",L_originBkg->color());
	krConfig->writeEntry("Origin Text",L_originText->color());
	krConfig->writeEntry("Status Background",L_statusBkg->color());
	krConfig->writeEntry("Status Text",L_statusText->color());
  krConfig->writeEntry("Menu Font",*L_menuFont);
  krConfig->writeEntry("Filelist Font",*L_fileListFont);
  krConfig->writeEntry("Cmdline Font",*L_cmdLineFont);	
  krConfig->writeEntry("History Font",*L_historyFont);
  krConfig->writeEntry("FnKeys Font",*L_fnKeysFont);	
  krConfig->writeEntry("Bookmark Font",*L_bookmarkFont);	
  // write settings for the External group
  krConfig->setGroup("External");
  krConfig->writeEntry("Terminal",E_terminal->text());
  krConfig->writeEntry("Editor",E_editor->text());
  krConfig->writeEntry("Terminal Exec",E_execSwitch->text());
  krConfig->writeEntry("Do Tar",E_doTAR->isChecked());
  krConfig->writeEntry("Do GZip",E_doGZIP->isChecked());
  krConfig->writeEntry("Do Zip",E_doZIP->isChecked());
  krConfig->writeEntry("Do UnZip",E_doUnZIP->isChecked());
  krConfig->writeEntry("Do Arj",E_doARJ->isChecked());
  krConfig->writeEntry("Do Rar",E_doRAR->isChecked());
  krConfig->writeEntry("Do BZip2",E_doBZIP2->isChecked());
  krConfig->writeEntry("Tar Path",E_tar->text());
  krConfig->writeEntry("GZip Path",E_gzip->text());
  krConfig->writeEntry("Zip Path",E_zip->text());
  krConfig->writeEntry("UnZip Path",E_unzip->text());
  krConfig->writeEntry("Arj Path",E_arj->text());
  krConfig->writeEntry("Rar Path",E_rar->text());
  krConfig->writeEntry("BZ2 Path",E_bzip2->text());
  // write settings for the Advanced group
  krConfig->setGroup("Advanced");
  krConfig->writeEntry("Icon Cache Size",A_iconCache->value());
  krConfig->writeEntry("Permission Check",A_permCheck->isChecked());
  krConfig->writeEntry("Confirm Unempty Dir",A_cUnemptyDir->isChecked());
  krConfig->writeEntry("Allow Move Into Archive",A_moveIntoArchive->isChecked());
  krConfig->writeEntry("Tab Interface",A_asTab->isChecked());
  krConfig->writeEntry("Tree Interface",A_asTree->isChecked());
  krConfig->writeEntry("AutoMount",A_autoMount->isChecked());

  accept();
}

// actually defaults all values
///////////////////////////////
void Konfigurator::slotApply() {
  // write settings for the General group
  krConfig->setGroup("General");
	krConfig->writeEntry("Mark Dirs",_MarkDirs);
	krConfig->writeEntry("Mimetype Magic",_MimetypeMagic);
	krConfig->writeEntry("Show Hidden",_ShowHidden);
	krConfig->writeEntry("Remember Position",_RememberPos);
	krConfig->writeEntry("Left Starts Tree",_LeftTree);
	krConfig->writeEntry("Right Starts Tree",_RightTree);	
  krConfig->writeEntry("Show Splashscreen",_SplashScreen);
	krConfig->writeEntry("Left Origin",_LeftOrigin);
	krConfig->writeEntry("Right Origin",_RightOrigin);
	// write settings for the LookNFeel group
  krConfig->setGroup("Look&Feel");
	krConfig->writeEntry("Origin Background",_OriginBkg);
	krConfig->writeEntry("Origin Text",_OriginText);
	krConfig->writeEntry("Status Background",_StatusBkg);
	krConfig->writeEntry("Status Text",_StatusText);
  krConfig->writeEntry("Menu Font",_MenuFont);
  krConfig->writeEntry("Filelist Font",_FilelistFont);
  krConfig->writeEntry("Cmdline Font",_CmdlineFont);	
  krConfig->writeEntry("History Font",_HistoryFont);
  krConfig->writeEntry("FnKeys Font",_FnKeysFont);	
  krConfig->writeEntry("Bookmark Font",_BookmarkFont);	
  // write settings for the External group
  krConfig->setGroup("External");
  krConfig->writeEntry("Terminal",_Terminal);
  krConfig->writeEntry("Editor",_Editor);
  krConfig->writeEntry("Terminal Exec",_TerminalExec);
  krConfig->writeEntry("Do Tar",_DoTar);
  krConfig->writeEntry("Do GZip",_DoGZip);
  krConfig->writeEntry("Do Zip",_DoZip);
  krConfig->writeEntry("Do Arj",_DoArj);
  krConfig->writeEntry("Do Rar",_DoRar);
  krConfig->writeEntry("Do BZip2",_DoBZip2);
  krConfig->writeEntry("Tar Path",_TarPath);
  krConfig->writeEntry("GZip Path",_GZipPath);
  krConfig->writeEntry("Zip Path",_ZipPath);
  krConfig->writeEntry("Arj Path",_ArjPath);
  krConfig->writeEntry("Rar Path",_RarPath);
  krConfig->writeEntry("BZ2 Path",_BZ2Path);
  // write settings for the Advanced group
  krConfig->setGroup("Advanced");
  krConfig->writeEntry("Icon Cache Size",_IconCacheSize);
  krConfig->writeEntry("Permission Check",_PermCheck);
  krConfig->writeEntry("Confirm Unempty Dir",_ConfirmUnemptyDir);
  krConfig->writeEntry("Allow Move Into Archive",_MoveIntoArchive);
  krConfig->writeEntry("Tab Interface",_TabInterface);
  krConfig->writeEntry("Tree Interface",_TreeInterface);
  krConfig->writeEntry("AutoMount",_AutoMount);
  krConfig->setGroup("Private");
  accept();
}

void Konfigurator::slotCancel() {
  reject();
}

void Konfigurator::leftPanelBrowse() {
  dirBrowser = new KChooseDir( 0,i18n("Select the left panel's homepage:"),
                               G_leftOrigin->text() );
	dirBrowser->setCaption(i18n("Start at..."));
	dirBrowser->exec();
	QString dest=dirBrowser->answear;
	if(dest==QString::null) return; //the usr canceled
  if (dest!="/")
    if (dest.right(1)=="/") dest.truncate(dest.length()-1);
  G_leftOrigin->setText(dest);
}

void Konfigurator::rightPanelBrowse() {
  dirBrowser = new KChooseDir( 0,i18n("Select the right panel's homepage:"),
                               G_rightOrigin->text() );
	dirBrowser->setCaption(i18n("Start at..."));
	dirBrowser->exec();
	QString dest=dirBrowser->answear;
	if(dest==QString::null) return; //the usr canceled
  if (dest!="/")
    if (dest.right(1)=="/") dest.truncate(dest.length()-1);
  G_rightOrigin->setText(dest);
}

void genericFontBrowse(QFont *&font, QLabel *&label) {
  int ok;
  QFont f;
  ok=KFontDialog::getFont(f);
  if ( ok!=1 ) return;  // the user cancelled
  delete font;  // recreate the font
  font=new QFont(f);
  QString temp=fontType(font);
  label->setFont(*font);
  label->setText(temp);
}

void Konfigurator::menuBarFontBrowse() {
  genericFontBrowse(L_menuFont,L_menuFontD);
}

void Konfigurator::fileListFontBrowse(){
  genericFontBrowse(L_fileListFont,L_fileListFontD);
}

void Konfigurator::cmdLineFontBrowse(){
  genericFontBrowse(L_cmdLineFont,L_cmdLineFontD);
}

void Konfigurator::historyFontBrowse(){
  genericFontBrowse(L_historyFont,L_historyFontD);
}

void Konfigurator::fnKeysFontBrowse(){
  genericFontBrowse(L_fnKeysFont,L_fnKeysFontD);
}

void Konfigurator::bookmarkFontBrowse(){
  genericFontBrowse(L_bookmarkFont,L_bookmarkFontD);
}

void genericFileBrowse(QLineEdit *&line, QString text) {
  QString name=KFileDialog::getOpenFileName ("/",0,0,text);
  if (!name) return;
  line->setText(name);
}

void Konfigurator::editorBrowse() {
  genericFileBrowse(E_editor,"My editor is");
}

void Konfigurator::terminalBrowse() {
  genericFileBrowse(E_terminal,"My terminal is");
}

void Konfigurator::tarBrowse() {
  genericFileBrowse(E_tar,"Handle TAR using");
}

void Konfigurator::gzipBrowse() {
  genericFileBrowse(E_gzip,"Open GZIP with");
}

void Konfigurator::bzip2Browse() {
  genericFileBrowse(E_bzip2,"I like my BZIP2 with");
}

void Konfigurator::zipBrowse() {
  genericFileBrowse(E_zip,"I pack my ZIPs with");
}

void Konfigurator::unzipBrowse() {
  genericFileBrowse(E_unzip,"Unpack ZIPs with");
}

void Konfigurator::rarBrowse() {
  genericFileBrowse(E_rar,"Open RAR files using");
}

void Konfigurator::arjBrowse() {
  genericFileBrowse(E_arj,"I prefer my ARJs with");
}

void Konfigurator::checkZipStatus() {
  if (E_doUnZIP->isChecked()) E_doZIP->setEnabled(true);
  else E_doZIP->setEnabled(false);
}

// This function fills the Janus widget with data
/////////////////////////////////////////////////
void Konfigurator::createLayout() {
  general=widget->addPage(i18n("General"),i18n("The every-day aspects of Krusader"));
  looknfeel=widget->addPage(i18n("Look & Feel"),i18n("The visuals"));
  external=widget->addPage(i18n("External"),i18n("Krusader's connections to the outside world"));
  advanced=widget->addPage(i18n("Advanced"),i18n("Don't touch unless you know what you're doing"));
  information=widget->addPage(i18n("Information"),i18n("Well, ehh... Information"));
  createGeneral();
  createLookNFeel();
  createExternal();
  createAdvanced();
  widget->showPage(General);
}

void Konfigurator::createGeneral() {
  QGridLayout *layout=new QGridLayout(general,1,1);
  ///////////////// group 1 ///////////////////////
  QGroupBox *group1=new QGroupBox(2,Qt::Vertical,i18n("Behavior"), general);
  group1->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
  G_markDirs=new QCheckBox(i18n("Auto marking selects directories"),group1);
    krConfig->setGroup("General");
    G_markDirs->setChecked(krConfig->readBoolEntry("Mark Dirs",_MarkDirs));
    QWhatsThis::add(G_markDirs,i18n("If checked, then SELECTing ALL or SELECTing GROUPs will mark directories"));
  G_useMagic=new QCheckBox(i18n("Use mimetype magic (slower)"),group1);
    G_useMagic->setChecked(krConfig->readBoolEntry("Mimetype Magic",_MimetypeMagic));
    QWhatsThis::add(G_useMagic,i18n("Mimetype magic allows the Krusader to \"peek\" into files and find out their type. This allows Krusader to display MUCH more different icons, but is a bit slower."));
  G_showHidden=new QCheckBox(i18n("Show hidden files"),group1);
    G_showHidden->setChecked(krConfig->readBoolEntry("Show Hidden",_ShowHidden));
  G_rememberPos=new QCheckBox(i18n("Remember position"),group1);
    G_rememberPos->setChecked(krConfig->readBoolEntry("Remember Position",_RememberPos));
    QWhatsThis::add(G_rememberPos,i18n("If checked, Krusader will remember it's size and position when closed. The next time it is run, Krusader will move to the same position and size."));
  ///////////////// group 2 ///////////////////////
  QGroupBox *group2=new QGroupBox(3,Qt::Horizontal,i18n("Start up"), general);
  group2->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
  new QLabel(i18n("Left panel homepage is "),group2);
  G_leftOrigin=new QLineEdit(krConfig->readEntry("Left Origin",_LeftOrigin),group2);
  G_leftOriginBrowse=new QPushButton(group2);
  G_leftOriginBrowse->setPixmap(folder_open_xpm);
  	G_leftOriginBrowse->setFixedWidth(10+G_leftOriginBrowse->pixmap()->width());
    connect(G_leftOriginBrowse,SIGNAL(clicked()),this,SLOT(leftPanelBrowse()));
  new QLabel(i18n("Right panel starts up at "),group2);
  G_rightOrigin=new QLineEdit(krConfig->readEntry("Right Origin",_RightOrigin),group2);
  G_rightOriginBrowse=new QPushButton(group2);
    G_rightOriginBrowse->setPixmap(folder_open_xpm);
  	G_leftOriginBrowse->setFixedWidth(10+G_leftOriginBrowse->pixmap()->width());
    connect(G_rightOriginBrowse,SIGNAL(clicked()),this,SLOT(rightPanelBrowse()));
  QGroupBox *miscGroup=new QGroupBox(1,Qt::Horizontal,i18n("Misc"),group2);
  miscGroup->setFrameStyle(QFrame::Box | QFrame::Plain);
    G_splashScreen=new QCheckBox(i18n("Show splashscreen"),miscGroup);
    G_splashScreen->setChecked(krConfig->readBoolEntry("Show Splashscreen",_SplashScreen));
  G_whosTheTree=new QButtonGroup(3,Qt::Vertical,i18n("Tree Panel"), group2);
    G_whosTheTree->setFrameStyle(QFrame::Box | QFrame::Plain);
    G_leftTree=new QRadioButton(i18n("Left panel starts as tree"),G_whosTheTree);
    G_rightTree=new QRadioButton(i18n("Right panel starts as tree"),G_whosTheTree);
    G_noTree=new QRadioButton(i18n("No trees on startup"),G_whosTheTree);
    G_leftTree->setChecked(krConfig->readBoolEntry("Left Starts Tree",_LeftTree));
    G_rightTree->setChecked(krConfig->readBoolEntry("Right Starts Tree",_RightTree));
    G_noTree->setChecked(!G_leftTree->isChecked() && !G_rightTree->isChecked());
  layout->addWidget(group1,0,0);
  layout->addWidget(group2,1,0);
}

void Konfigurator::createLookNFeel() {
  QGridLayout *layout=new QGridLayout(looknfeel,1,1);
  QFontMetrics *fontData;
  krConfig->setGroup("Look&Feel");
  ///////////////// group 1 ///////////////////////
  QGroupBox *group1=new QGroupBox(3,Qt::Horizontal,i18n("Fonts"),looknfeel);
  group1->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
  new QLabel(i18n("Menubar font:"),group1);
    L_menuFont=new QFont(krConfig->readFontEntry("Menu Font",_MenuFont));
    QString temp=fontType(L_menuFont);
    L_menuFontD=new QLabel(group1);
    L_menuFontD->setText(temp);
    L_menuFontD->setFont(*L_menuFont);
    L_menuFontD->setMaximumWidth(MAXWIDTH);
    L_menuFontB=new QPushButton(i18n("Change"),group1);
  	fontData=new QFontMetrics(L_menuFontB->font());
  	L_menuFontB->setFixedWidth(20+fontData->width(L_menuFontB->text()));
  	delete fontData;
    connect(L_menuFontB,SIGNAL(clicked()),this,SLOT(menuBarFontBrowse()));
  new QLabel(i18n("Filelist font:"),group1);
    L_fileListFont=new QFont(krConfig->readFontEntry("Filelist Font",_FilelistFont));
    temp=fontType(L_fileListFont);
    L_fileListFontD=new QLabel(group1);
    L_fileListFontD->setText(temp);
    L_fileListFontD->setFont(*L_fileListFont);
    L_fileListFontD->setMaximumWidth(MAXWIDTH);
    L_fileListFontB=new QPushButton(i18n("Change"),group1);
  	fontData=new QFontMetrics(L_fileListFontB->font());
  	L_fileListFontB->setFixedWidth(20+fontData->width(L_fileListFontB->text()));
  	delete fontData;
    connect(L_fileListFontB,SIGNAL(clicked()),this,SLOT(fileListFontBrowse()));
 	new QLabel(i18n("Command line font:"),group1);
    L_cmdLineFont=new QFont(krConfig->readFontEntry("Cmdline Font",_CmdlineFont));
    temp=fontType(L_cmdLineFont);
    L_cmdLineFontD=new QLabel(group1);
    L_cmdLineFontD->setText(temp);
    L_cmdLineFontD->setFont(*L_cmdLineFont);
    L_cmdLineFontD->setMaximumWidth(MAXWIDTH);
    L_cmdLineFontB=new QPushButton(i18n("Change"),group1);
  	fontData=new QFontMetrics(L_cmdLineFontB->font());
  	L_cmdLineFontB->setFixedWidth(20+fontData->width(L_cmdLineFontB->text()));
  	delete fontData;
    connect(L_cmdLineFontB,SIGNAL(clicked()),this,SLOT(cmdLineFontBrowse()));
	new QLabel(i18n("History list font:"),group1);
    L_historyFont=new QFont(krConfig->readFontEntry("History Font",_HistoryFont));
    temp=fontType(L_historyFont);
    L_historyFontD=new QLabel(group1);
    L_historyFontD->setText(temp);
    L_historyFontD->setFont(*L_historyFont);
    L_historyFontD->setMaximumWidth(MAXWIDTH);
    L_historyFontB=new QPushButton(i18n("Change"),group1);
    fontData=new QFontMetrics(L_historyFontB->font());
  	L_historyFontB->setFixedWidth(20+fontData->width(L_historyFontB->text()));
  	delete fontData;
    connect(L_historyFontB,SIGNAL(clicked()),this,SLOT(historyFontBrowse()));
	new QLabel(i18n("Function keys font:"),group1);
    L_fnKeysFont=new QFont(krConfig->readFontEntry("FnKeys Font",_FnKeysFont));
    temp=fontType(L_fnKeysFont);
    L_fnKeysFontD=new QLabel(group1);
    L_fnKeysFontD->setText(temp);
    L_fnKeysFontD->setFont(*L_fnKeysFont);
    L_fnKeysFontD->setMaximumWidth(MAXWIDTH);
    L_fnKeysFontB=new QPushButton(i18n("Change"),group1);
    fontData=new QFontMetrics(L_fnKeysFontB->font());
  	L_fnKeysFontB->setFixedWidth(20+fontData->width(L_fnKeysFontB->text()));
  	delete fontData;
    connect(L_fnKeysFontB,SIGNAL(clicked()),this,SLOT(fnKeysFontBrowse()));
	new QLabel(i18n("Bookmarks font:"),group1);
    L_bookmarkFont=new QFont(krConfig->readFontEntry("Bookmark Font",_BookmarkFont));
    temp=fontType(L_bookmarkFont);
    L_bookmarkFontD=new QLabel(group1);
    L_bookmarkFontD->setText(temp);
    L_bookmarkFontD->setFont(*L_bookmarkFont);
    L_bookmarkFontD->setMaximumWidth(MAXWIDTH);
    L_bookmarkFontB=new QPushButton(i18n("Change"),group1);
    fontData=new QFontMetrics(L_bookmarkFontB->font());
  	L_bookmarkFontB->setFixedWidth(20+fontData->width(L_bookmarkFontB->text()));
  	delete fontData;
    connect(L_bookmarkFontB,SIGNAL(clicked()),this,SLOT(bookmarkFontBrowse()));
  ///////////////// group 2 ///////////////////////
  QGroupBox *group2=new QGroupBox(4,Qt::Horizontal,i18n("Active Panel"),looknfeel);
  group2->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
	new QLabel(i18n("Path button color:"),group2);
	  L_originText=new KColorButton(krConfig->readColorEntry("Origin Text",_OriginText),group2);
	  new QLabel(i18n("background:"),group2);
    L_originBkg=new KColorButton(krConfig->readColorEntry("Origin Background",_OriginBkg),group2);	
	new QLabel(i18n("Status lines color:"),group2);
	  L_statusText=new KColorButton(krConfig->readColorEntry("Status Text",_StatusText),group2);
	  new QLabel(i18n("background:"),group2);
    L_statusBkg=new KColorButton(krConfig->readColorEntry("Status Background",_StatusBkg),group2);	

  layout->addWidget(group1,0,0);
  layout->addWidget(group2,1,0);
}

void Konfigurator::createExternal() {
  QGridLayout *layout=new QGridLayout(external,1,1);
  krConfig->setGroup("External");
  ///////////////// group 1 ///////////////////////
  QGroupBox *group1=new QGroupBox(3,Qt::Horizontal,i18n("General"),external);
  group1->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
  new QLabel(i18n("Krusader's editor"),group1);
  E_editor=new QLineEdit(krConfig->readEntry("Editor",_Editor),group1);
  QWhatsThis::add(E_editor,i18n("When asking Krusader to EDIT a file, it invokes the editor listed here."));
  E_editorBrowse=new QPushButton(group1);
    E_editorBrowse->setPixmap(folder_open_xpm);
  	E_editorBrowse->setFixedWidth(10+E_editorBrowse->pixmap()->width());
    connect(E_editorBrowse,SIGNAL(clicked()),this,SLOT(editorBrowse()));
  new QLabel(i18n("Krusader's terminal"),group1);
  E_terminal=new QLineEdit(krConfig->readEntry("Terminal",_Terminal),group1);
  E_terminalBrowse=new QPushButton(group1);
    E_terminalBrowse->setPixmap(folder_open_xpm);
  	E_terminalBrowse->setFixedWidth(10+E_terminalBrowse->pixmap()->width());
    connect(E_terminalBrowse,SIGNAL(clicked()),this,SLOT(terminalBrowse()));
  new QLabel(i18n("  Terminal's exec switch"),group1);
  E_execSwitch=new QLineEdit(krConfig->readEntry("Terminal Exec",_TerminalExec),group1);
  QWhatsThis::add(E_execSwitch,i18n("Each terminal has an execute switch, most of them use '-e'. If yours doesn't, enter the correct switch."));
  group1->addSpace(0);
  //////////////// group 2 ///////////////////////
  QGroupBox *group2=new QGroupBox(3,Qt::Horizontal,i18n("Archives"),external);
  group2->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
  E_doTAR=new QCheckBox(i18n("Handle Tar using"),group2);
    E_doTAR->setChecked(krConfig->readBoolEntry("Do Tar",_DoTar));
    E_tar=new QLineEdit(krConfig->readEntry("Tar Path",_TarPath),group2);
    E_tarBrowse=new QPushButton(group2);
    E_tarBrowse->setPixmap(folder_open_xpm);
  	E_tarBrowse->setFixedWidth(10+E_tarBrowse->pixmap()->width());
    connect(E_tarBrowse,SIGNAL(clicked()),this,SLOT(tarBrowse()));
  E_doGZIP=new QCheckBox(i18n("Handle GZip using"),group2);
    E_doGZIP->setChecked(krConfig->readBoolEntry("Do GZip",_DoGZip));
    E_gzip=new QLineEdit(krConfig->readEntry("GZip Path",_GZipPath),group2);
    E_gzipBrowse=new QPushButton(group2);
    E_gzipBrowse->setPixmap(folder_open_xpm);
  	E_gzipBrowse->setFixedWidth(10+E_gzipBrowse->pixmap()->width());
    connect(E_gzipBrowse,SIGNAL(clicked()),this,SLOT(gzipBrowse()));
  E_doBZIP2=new QCheckBox(i18n("Handle BZip2 using"),group2);
    E_doBZIP2->setChecked(krConfig->readBoolEntry("Do BZip2",_DoBZip2));
    E_bzip2=new QLineEdit(krConfig->readEntry("BZ2 Path",_BZ2Path),group2);
    E_bzip2Browse=new QPushButton(group2);
    E_bzip2Browse->setPixmap(folder_open_xpm);
  	E_bzip2Browse->setFixedWidth(10+E_bzip2Browse->pixmap()->width());
    connect(E_bzip2Browse,SIGNAL(clicked()),this,SLOT(bzip2Browse()));
  E_doUnZIP=new QCheckBox(i18n("Unpack Zip using"),group2);
    E_doUnZIP->setChecked(krConfig->readBoolEntry("Do UnZip",_DoUnZip));
    E_unzip=new QLineEdit(krConfig->readEntry("UnZip Path",_UnZipPath),group2);
    E_unzipBrowse=new QPushButton(group2);
    E_unzipBrowse->setPixmap(folder_open_xpm);
  	E_unzipBrowse->setFixedWidth(10+E_unzipBrowse->pixmap()->width());
    connect(E_unzipBrowse,SIGNAL(clicked()),this,SLOT(unzipBrowse()));
    connect(E_doUnZIP,SIGNAL(clicked()),this,SLOT(checkZipStatus()));
  E_doZIP=new QCheckBox(i18n("Pack to Zip using"),group2);
    E_doZIP->setChecked(krConfig->readBoolEntry("Do Zip",_DoZip));
    E_zip=new QLineEdit(krConfig->readEntry("Zip Path",_ZipPath),group2);
    E_zipBrowse=new QPushButton(group2);
    E_zipBrowse->setPixmap(folder_open_xpm);
  	E_zipBrowse->setFixedWidth(10+E_zipBrowse->pixmap()->width());
    connect(E_zipBrowse,SIGNAL(clicked()),this,SLOT(zipBrowse()));
  E_doARJ=new QCheckBox(i18n("Handle Arj using"),group2);
    E_doARJ->setChecked(krConfig->readBoolEntry("Do Arj",_DoArj));
    E_arj=new QLineEdit(krConfig->readEntry("Arj Path",_ArjPath),group2);
    E_arjBrowse=new QPushButton(group2);
    E_arjBrowse->setPixmap(folder_open_xpm);
  	E_arjBrowse->setFixedWidth(10+E_arjBrowse->pixmap()->width());
    connect(E_arjBrowse,SIGNAL(clicked()),this,SLOT(arjBrowse()));
  E_doRAR=new QCheckBox(i18n("Handle Rar using"),group2);
    E_doRAR->setChecked(krConfig->readBoolEntry("Do Rar",_DoRar));
    E_rar=new QLineEdit(krConfig->readEntry("Rar Path",_RarPath),group2);
    E_rarBrowse=new QPushButton(group2);
    E_rarBrowse->setPixmap(folder_open_xpm);
  	E_rarBrowse->setFixedWidth(10+E_rarBrowse->pixmap()->width());
    connect(E_rarBrowse,SIGNAL(clicked()),this,SLOT(rarBrowse()));

  layout->addWidget(group1,0,0);
  layout->addWidget(group2,1,0);
}

void Konfigurator::createAdvanced() {
  QGridLayout *layout=new QGridLayout(advanced,1,1);
  krConfig->setGroup("Advanced");
  ///////////////// group 1 ///////////////////////
  QGroupBox *group1=new QGroupBox(2,Qt::Horizontal,i18n("Power-ups"),advanced);
  group1->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
  A_permCheck=new QCheckBox(i18n("Check permissions before doing anything"),group1);
    A_permCheck->setChecked(krConfig->readBoolEntry("Permission Check",_PermCheck));
    QWhatsThis::add(A_permCheck,i18n("If checked, Krusder will check the file's permission before trying to delete it, rename it etc. You might not want this: Even if you're ROOT (Krusader favours ROOTs) and CAN DO ANYTHING, Krusader won't let you delete a file if the permission says so !"));
  A_moveIntoArchive=new QCheckBox(i18n("Allow move into archives"),group1);
    A_moveIntoArchive->setChecked(krConfig->readBoolEntry("Allow Move Into Archive",_MoveIntoArchive));
    QWhatsThis::add(A_moveIntoArchive,i18n("If checked, Krusader will allow you to move files into archive. This is a bit tricky, as a system failure while moving might result in misplaced files. IMPORTANT: The files will not be deleted, just misplaced into a directory in /tmp, you'll need to search for them, incase a disaster strikes."));
  A_cUnemptyDir=new QCheckBox(i18n("Warn before deleting a non-empty directory"),group1);
    A_cUnemptyDir->setChecked(krConfig->readBoolEntry("Confirm Unempty Dir",_ConfirmUnemptyDir));
    QWhatsThis::add(A_cUnemptyDir,i18n("If checked, Krusder will ask before deleting a non-empty directory. Notice, however, that the question is asked once (!) for the parent directory. If you said YES, Krusader will delete everything underneath."));
  A_autoMount=new QCheckBox(i18n("Automount non-mounted points"),group1);
    A_autoMount->setChecked(krConfig->readBoolEntry("AutoMount",_AutoMount));
    QWhatsThis::add(A_autoMount,i18n("If checked and you try to enter a directory which appears as a mount point in /etc/fstab, Krusader will attempt to automount it for you."));
  /////////////////// group 2 //////////////////////
  QGroupBox *group2=new QGroupBox(2,Qt::Horizontal,i18n("Krusader Internals"),advanced);
  group2->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
  new QLabel(i18n("Krusader's icon cache size (KB)"),group2);
  A_iconCache=new QSpinBox(0, 8192, 1,group2);
    A_iconCache->setValue(krConfig->readNumEntry("Icon Cache Size",_IconCacheSize));
    QWhatsThis::add(A_iconCache,i18n("Cache size determines how fast can Krusader display the contents of a panel. However (and as the size is in KBs), too big a cache might consume your memory."));
  A_konfigurator=new QButtonGroup(2,Qt::Vertical,i18n("Konfigurator's Interface"), group2);
    A_asTab=new QRadioButton(i18n("I like my Konfigurator tabbed."),A_konfigurator);
    A_asTab->setChecked(krConfig->readBoolEntry("Tab Interface",_TabInterface));
    A_asTree=new QRadioButton(i18n("I prefer my Konfigurator as a tree."),A_konfigurator);
    A_asTree->setChecked(krConfig->readBoolEntry("Tree Interface",_TreeInterface));
  layout->addWidget(group1,0,0);
  layout->addWidget(group2,1,0);
}
