/***************************************************************************
                                krusaderview.cpp
                             -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description
 ***************************************************************************

  A

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "krusaderview.h"
#include "krusader.h"
#include "defaults.h"

KrusaderView::KrusaderView(QWidget *parent, const char *name ) : QWidget(parent,name) {
  ////////////////////////////////
  // make a 1x1 mainLayout, it will auto-expand:
	mainLayout = new QGridLayout(this, 1, 1);
	// create a command line thing
  cmdLine=new KCMDLine(this);  			
	// and the splitter...
	splitter = new QSplitter( this );
  // add 2 pseudo widgets and layouts to allow a smoother movement of the
  // whole screen, and allow the status bars to resize with the panels
  krConfig->setGroup("General");  // this is needed !
  if (!krConfig->readBoolEntry("Left Starts Tree",_LeftTree))
    left=new ListPanel(splitter,false);	else left=new TreePanel(splitter);
  krConfig->setGroup("General");  // this is needed too
  if (!krConfig->readBoolEntry("Right Starts Tree",_RightTree))
    right=new ListPanel(splitter,true); else right=new TreePanel(splitter);
  left->setOther(right); right->setOther(left);
  // make the left panel focused at program start
  right->start(false); left->start(true);
  activePanel=left; activePanel->slotFocusOnMe();  // left starts out active
	splitter->resize(splitter->sizeHint());
  // create the function keys widget
	fnKeys=new KFnKeys(this);
	fnKeys->show();
	bindKeys();
	// and insert the whole thing into the main layout... at last	
	mainLayout->addWidget(splitter,0,0);
	mainLayout->addWidget(cmdLine,1,0);
	mainLayout->addWidget(fnKeys,2,0);
	mainLayout->activate();
}

void KrusaderView::bindKeys() {
  keys=new KAccel(this);
  // Tab - panel switch
  keys->insertItem(i18n("Switch panel"), "Switch panel", Key_Tab);
  keys->connectItem("Switch panel", this , SLOT(panelSwitch()));
  // CTRL-R - force refresh
  keys->insertItem(i18n("Refresh panel"), "Refresh panel", CTRL+Key_R);
  keys->connectItem("Refresh panel", krApp, SLOT(refresh()));
}

// updates the command line whenever current panel changes
//////////////////////////////////////////////////////////
void KrusaderView::slotCurrentChanged(QString p) { cmdLine->setCurrent(p); }

void KrusaderView::setListView() {
  KVFSPanel *temp;

  if (activePanel->type == "list") return; // if the panel is a list-panel, exit
  // otherwise, we first kill the active panel and recreate it, saving pointers
  if (activePanel==left) {
	  hide();
	  // keep the right immobile
	  splitter->setResizeMode(right,QSplitter::KeepSize);
    temp=left;
    // create a new panel
    left = new ListPanel(splitter,false);
    left->otherPanel=right;
    right->otherPanel=left;
    activePanel=left;
    left->start(true);
    // move the new one to the left-most position
    splitter->moveToFirst(left);
 	  splitter->setResizeMode(left,QSplitter::KeepSize);
    // and kill the old one
    delete temp;
    splitter->refresh();
    left->slotFocusOnMe();
    show();
    return;
  }
  if (activePanel==right) {
	  hide();
	  // keep the left immobile
	  splitter->setResizeMode(left,QSplitter::KeepSize);
    temp=right;
    // create a new panel
    right = new ListPanel(splitter,true);
    right->otherPanel=left;
    left->otherPanel=right;
    activePanel=right;
    right->start(false);
    // move the new one to the right-most position
    splitter->moveToLast(right);
 	  splitter->setResizeMode(right,QSplitter::KeepSize);
    // and kill the old one
    delete temp;
    splitter->refresh();
    right->slotFocusOnMe();
    show();
    return;
  }
}

void KrusaderView::setTreeView() {
  KVFSPanel *temp;

  if (activePanel->type == "tree") return; // if the panel is a tree-view, exit
  if (activePanel->otherPanel->type == "tree") return;  // we can't have 2 trees!!!
  // otherwise, we first kill the active panel and recreate it, saving pointers
  if (activePanel==left) {
	  hide();
	  // keep the right immobile
	  splitter->setResizeMode(right,QSplitter::KeepSize);
    temp=left;
    // create a new panel
    left = new TreePanel(splitter);
    left->otherPanel=right;
    right->otherPanel=left;
    activePanel=left;
    left->start(true);
    // move the new one to the left-most position
    splitter->moveToFirst(left);
 	  splitter->setResizeMode(left,QSplitter::KeepSize);
    // and kill the old one
    delete temp;
    splitter->refresh();
    left->slotFocusOnMe();
    show();
    return;
  }
  if (activePanel==right) {
	  hide();
	  // keep the left immobile
	  splitter->setResizeMode(left,QSplitter::KeepSize);
    temp=right;
    // create a new panel
    right = new TreePanel(splitter);
    right->otherPanel=left;
    left->otherPanel=right;
    activePanel=right;
    right->start(false);
    // move the new one to the right-most position
    splitter->moveToLast(right);
 	  splitter->setResizeMode(right,QSplitter::KeepSize);
    // and kill the old one
    delete temp;
    splitter->refresh();
    right->slotFocusOnMe();
    show();
    return;
  }
}
