/***************************************************************************
                                listpanel.cpp
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <kapp.h>
#include <klocale.h>
#include <kmimetype.h>
#include <kfileitem.h>
#include <kurl.h>
#include <ktrader.h>
#include <krun.h>
#include <kopenwith.h>
// Krusader includes
#include "kvfspanel.h"
#include "krlistitem.h"
#include "krusader.h"
#include "resources.h"
#include "defaults.h"
#include "kicons.h"
// QT includes
#include <qbitmap.h>
#include <qwhatsthis.h>
#include <qstringlist.h>
#include <qstrlist.h>
#include <qdragobject.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>

/////////////////////////////////////////////////////
// 					The list panel constructor             //
/////////////////////////////////////////////////////
ListPanel::ListPanel(QWidget *parent, const bool mirrored, const char *name ) :
					 KVFSPanel(parent,name), t(0), currDragItem(0) {

  type = "list";
	func = new ListPanelFunc(this);
  setAcceptDrops(true);
  setMouseTracking(false);
	layout=new QGridLayout(this,3,2);
  	// ... create the bookmark list
  bookmarkList=new QToolButton(this);
  bookmarkList->setPixmap(flag_xpm);
  bookmarkList->setMaximumWidth(32);
  bookmarkList->setTextLabel(i18n("Open your bookmarks"),true);
  bookmarkList->setPopupDelay(100); // 0.1 seconds press
  connect(bookmarkList,SIGNAL(pressed()),this,SLOT(slotFocusOnMe()));
  connect(bookmarkList,SIGNAL(pressed()),this,SLOT(slotRefreshBookmarks()));
    // create the pop-up menu for the bookmarks,
    // this means the panel needs to be
  bookmarks=new QPopupMenu(this);
  krConfig->setGroup("Look&Feel");
  bookmarks->setFont(krConfig->readFontEntry("Bookmark Font",_BookmarkFont));
  bookmarks->setPalette(QPalette( krConfig->readColorEntry("Status Background",_StatusBkg) ));
  QPalette* qp= const_cast<QPalette*>(&(bookmarks->palette()));
  qp->setColor(  QPalette::Normal,QColorGroup::Foreground,
                krConfig->readColorEntry("Status Text",_StatusText));
  slotRefreshBookmarks();
    // and connect it to the button
  bookmarkList->setPopup(bookmarks);
  QWhatsThis::add(bookmarkList,WT_PANEL_BOOKMARKS);
  connect(bookmarks,SIGNAL(activated(int)),this,
          SLOT(slotBookmarkChosen(int)));

  status = new QLabel(this);
  status->setFont(krConfig->readFontEntry("Bookmark Font",_BookmarkFont));
  status->setFrameStyle( QFrame::Box | QFrame::Raised);
  status->setLineWidth(1);		// a nice 3D touch :-)
  status->setText("");        // needed for initialization code!
  QWhatsThis::add(status,WT_PANEL_STATUS);

  totals = new QLabel(this);
  totals->setFont(krConfig->readFontEntry("Bookmark Font",_BookmarkFont));
  totals->setFrameStyle( QFrame::Box | QFrame::Raised);
  totals->setLineWidth(1);		// a nice 3D touch :-)
  QWhatsThis::add(totals,WT_PANEL_TOTALS);

	origin = new QPushButton("",this);
	krConfig->setGroup("Look&Feel");
	origin->setFont(krConfig->readFontEntry("Filelist Font",_FilelistFont));
	origin->font().setBold(true);
	origin->font().setPointSize(origin->font().pointSize()+2);
		// pressing the origin bar sets focus
  QWhatsThis::add(origin,WT_PANEL_ORIGIN);
	connect(origin,SIGNAL(clicked()),this,SLOT(slotFocusOnMe()));

  fileList = new KFileList(this);
  krConfig->setGroup("Look&Feel");
  fileList->setFont(krConfig->readFontEntry("Filelist Font",_FilelistFont));
	fileList->setAllColumnsShowFocus(true);
	fileList->setMultiSelection(true);
	fileList->setSelectionMode(QListView::Extended);
	fileList->setShowSortIndicator(true);
	fileList->setVScrollBarMode(QScrollView::AlwaysOn);
	fileList->setHScrollBarMode(QScrollView::Auto);
	int i=QFontMetrics(fileList->font()).width("W");
	int j=QFontMetrics(fileList->font()).width("0");
	j=(i>j ? i : j);
	int d=QFontMetrics(fileList->font()).width("09/09/09  09:09");
	fileList->addColumn(i18n("Name"),i*17);
	fileList->addColumn(i18n("Size"),j*7);
	fileList->setColumnAlignment(1,Qt::AlignRight);
	fileList->addColumn(i18n("Date"),d+5);
	fileList->setColumnAlignment(2,Qt::AlignHCenter);
	fileList->addColumn(i18n("r"),17);
	fileList->addColumn(i18n("w"),17);
	fileList->addColumn(i18n("x"),17);
  fileList->setColumnAlignment(3,Qt::AlignCenter);
  fileList->setColumnAlignment(4,Qt::AlignCenter);
  fileList->setColumnAlignment(5,Qt::AlignCenter);
  QWhatsThis::add(fileList,WT_PANEL_FILELIST);
    // when fileList wants us to drag ...
  connect(fileList, SIGNAL(letsDrag(int)), this, SLOT(startDragging(int)));	
    // when the drag is finished
  connect(this,SIGNAL(finishedDragging()), fileList, SLOT(finishedDragging()));
  	// double click and pressing return executes
  connect(fileList, SIGNAL(doubleClicked(QListViewItem*)), this,
          SLOT(slotExecuted(QListViewItem*)));
  connect(fileList, SIGNAL(returnPressed(QListViewItem *)), this,
          SLOT(slotExecuted(QListViewItem*)));
		// even a "drag-swush" through the list changes focus
	connect(fileList, SIGNAL(mouseButtonPressed(int,QListViewItem *,const QPoint &,int)),
					this,SLOT(slotFocusOnMe()));
		// mouse over item event
	connect(fileList, SIGNAL(onItem(QListViewItem*)),
					this,SLOT(slotChangeStatus(QListViewItem*)));
		// a change in the selection needs to update totals
	connect(fileList, SIGNAL(selectionChanged()), this,	SLOT(slotUpdateTotals()));
    // right-click menu
  connect(fileList, SIGNAL(rightClickMenu(QListViewItem*,QPoint)), this,
          SLOT(popRightClickMenu(QListViewItem*,QPoint)));
		// conncet krusder status bar with mouse over event...
	connect(this,SIGNAL(signalStatus(QString)),
					krApp, SLOT(statusBarUpdate(QString)));

		// make sure that a focus/path change reflects in the command line and activePanel
	connect(this,SIGNAL(cmdLineUpdate(QString)),this->parent()->parent(),
					SLOT(slotCurrentChanged(QString)) );
	connect(this,SIGNAL(activePanelChanged(KVFSPanel *)),this->parent()->parent(),
					SLOT(slotSetActivePanel(KVFSPanel *)) );

  	// connect the signal cmdlineupdate to make krusader update stat
	connect(this,SIGNAL(cmdLineUpdate(QString)),this,
					SLOT(slotGetStats(QString)) );
		
		// finish the layout
	layout->addMultiCellWidget(origin,0,0,0,1);
	if (mirrored) {
		layout->addWidget(status,1,0);
		layout->addWidget(bookmarkList,1,1);
		
	} else {
		layout->addWidget(status,1,1);
		layout->addWidget(bookmarkList,1,0);	
	}
	layout->addMultiCellWidget(fileList,2,2,0,1);
	layout->addMultiCellWidget(totals,3,3,0,1);

	vfsStack.setAutoDelete(false);
}

void ListPanel::cleanUp(){
  while(!vfsStack.isEmpty()){
    delete files;
    files = vfsStack.pop();
  }
}

// this is used to start the panel, AFTER setOther() has been used
//////////////////////////////////////////////////////////////////
void ListPanel::start(bool left) {
	krConfig->setGroup("General");
	//////////////////////////////
	left ? virtualPath=krConfig->readEntry("Left Origin",_LeftOrigin) :
	       virtualPath=krConfig->readEntry("Right Origin",_RightOrigin);
	
	QFileInfo qfi(virtualPath);
	if (!qfi.isDir() || !qfi.exists()){
	  virtualPath = "/";
	  QString side = (left ? "left" : "right");
	  QMessageBox mb( i18n("Error - Krusader"),
	      i18n("The "+side+" panel start path is invalid\nCheck your Konfigurator"),
	        QMessageBox::Warning,
	        QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape
	        ,0,0);
     mb.exec();
  }	
	
	realPath = virtualPath;
	
	files = new normal_vfs(virtualPath,this);
	files->vfs_refresh(virtualPath);
}

void ListPanel::slotStartUpdate(){
	QListViewItem *item;
  if (t!=0) delete t;
 	fileList->clear();

	// add the ".." entery
	if(files->vfs_getOrigin()!="/"){
		item=new KRListItem(fileList,"..","<DIR>");
		item->setPixmap(0,up_xpm);
	  item->setSelectable(false);
	  item->setPixmap(3,yes_xpm);
	  item->setPixmap(4,yes_xpm);
	  item->setPixmap(5,yes_xpm);
	}
	currentVfile = 0;
	
	t = new QTimer(this);
  connect( t,SIGNAL(timeout()),SLOT(slotUpdate()));
  t->start(0,FALSE);
  canStopTimer = false;
}

void ListPanel::slotUpdate(){
	QListViewItem *item;
	QString size;
		
	if( canStopTimer && currentVfile >= files->vfs_noOfFiles() ){
	  delete t;
	  t =0;
	}
	
	for(vfile* vf=files->vfs_at(currentVfile); vf!=0; vf=files->vfs_at(++currentVfile)){
	
	 size = parseSize(vf->vfile_getSize());
		
	  item = new KRListItem(fileList,vf->vfile_getName(),
				vf->vfile_isDir() ? QString("<DIR>") : size,
		    vf->vfile_getDateTime(),"","","");

    item->setPixmap(3,vf->vfile_isReadable() ? yes_xpm : no_xpm);
	  item->setPixmap(4,vf->vfile_isWriteable() ? yes_xpm : no_xpm);
	  item->setPixmap(5,vf->vfile_isExecutable() ? yes_xpm : no_xpm);
	  item->setPixmap(0,getIcon(vf));
	
	  if(currentVfile%25 == 0){
	    ++currentVfile;
	    break;
	  }
	}	
}

void ListPanel::slotEndUpdate(){	
	virtualPath = files->vfs_getOrigin();
	if(files->vfs_getType() == "normal")
	  realPath = virtualPath;

	slotUpdateTotals();
	origin->setText(shortPath());
	emit cmdLineUpdate(realPath);	// update the command line
  canStopTimer = true;
}

// walk through the list and update the no. of selected files and their size
////////////////////////////////////////////////////////////////////////////
void ListPanel::slotUpdateTotals() {
	int num=0;
	long size=0;
	QString total;
		
	QListViewItem *iterator=fileList->firstChild();
	while (iterator!=0) {
		if (iterator->isSelected() && iterator->text(0)!=QString("..")) {
			if ( (iterator->text(1))!=QString("<DIR>") )
			  size+=(iterator->text(1).replace(QRegExp(","),"").toLong());
			num++;
		}
		iterator=iterator->itemBelow();
	}

	total = QString("%1 "+i18n("out of")+" %2 "+i18n("selected")+", %3 "+i18n("out of")+
					" %4").arg(num).
					arg(files->vfs_noOfFiles()).arg(KIO::convertSize(size)).
					arg(KIO::convertSize(files->vfs_totalSize()));
	totals->setText(total);
}

QPixmap ListPanel::getIcon(vfile* vf){
	krConfig->setGroup("Advanced");
	//////////////////////////////
	QPixmap icon;
	QString mime = vf->vfile_getMime();
	
	QPixmapCache::setCacheLimit( krConfig->readNumEntry("Icon Cache Size",_IconCacheSize) );
	
	// first try the cache
	if (!QPixmapCache::find(mime,icon)){
	  // get the icon.
	  QIconSet set( KMimeType::mimeType(mime)->pixmap(0) );//,KIcon::SizeSmall);
    // resize the icon
    icon = set.pixmap(QIconSet::Small,true);
	  // insert it into the cache
    QPixmapCache::insert(mime,icon);
	}
	// if it's a symlink - add an arrow overlay
	if(vf->vfile_isSymLink()){
		QPixmap link(link_xpm);
		bitBlt (&icon,0,11,&link,0,21,10,11,CopyROP,false);
		icon.setMask( icon.createHeuristicMask(false) );
  }
	
	return icon;	
}

void ListPanel::slotChangeStatus(QListViewItem *i){
	if (i->text(0)=="..") emit signalStatus(i18n("Climb up the directory tree"));
	else{
 	   	vfile* vf = files->vfs_search(i->text(0));
			if(vf == 0) return;
			
			QString text = vf->vfile_getName();
			QString comment = KMimeType::mimeType(vf->vfile_getMime() )->comment(text, false );
  		
  		QString myLinkDest = vf->vfile_getSymDest();
  		long mySize = vf->vfile_getSize();

 			QString text2 = text.copy();
  		mode_t m_fileMode = vf->vfile_getMode();

  		if (vf->vfile_isSymLink() ){
      	QString tmp;
      	
				if ( comment.isEmpty() )	tmp = i18n ( "Symbolic Link" );
        else  tmp = i18n("%1 (Link)").arg(comment);
      	
				text += "->";
      	text += myLinkDest;
      	text += "  ";
      	text += tmp;
  		} else if ( S_ISREG( m_fileMode ) ){
      	text = QString("%1 (%2)").arg(text2).arg( KIO::convertSize( mySize ) );
      	text += "  ";
      	text += comment;
  		} else if ( S_ISDIR ( m_fileMode ) ){
      	text += "/  ";
     		text += comment;
    	} else {
      	text += "  ";
      	text += comment;
    	}
			
		emit signalStatus(text);
	}
}


//  refresh the panel when a bookmark was chosen
////////////////////////////////////////////////
void ListPanel::slotBookmarkChosen(int id) {
 	krConfig->setGroup("Bookmarks");
 	////////////////////////////////
 	QStringList list;
 	list=krConfig->readListEntry("Links");
  if (list.count()==0) list.append(_Links);
  if (krConfig->readBoolEntry("Sorted",_Sorted)) list.sort();
  QStringList::Iterator it=list.begin();
  for (int i=0; i<id; ++i) ++it;
 	slotPanelRefresh(*it);
}

void ListPanel::slotRefreshBookmarks() {
  krConfig->setGroup("Bookmarks");
  ////////////////////////////////
  QStringList list=krConfig->readListEntry("Links");
  if (list.count()==0) list.append(_Links);
  if (krConfig->readBoolEntry("Sorted",_Sorted)) list.sort();
  bookmarks->clear();   // just in case
  int i=0;
  for(QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
    bookmarks->insertItem(*it,i);
    ++i;
  }
}

void ListPanel::slotGetStats(QString path) {
  if (status->text()!="") return;   // don't update when not needed
  // 1st, get information
  if (!krMtMan.operational()) {       // don't try if no mountman !! :-(
    status->setText("Mt.Man is dead (sorry). Check existance of /bin/df");
    return;
  }
  while (!krMtMan.ready());           // poll until mt.man is ready
  fsData temp=krMtMan.getData(path);
  QString templabel,label;
  if (path=="/proc") { // /proc is a special case - no volume information
    QString label;
    label="[No space information] on [ "+temp.mntPoint()+" ("+temp.type()+") ]";
    status->setText(label);
    return;
  }
  templabel=KIO::convertSize(temp.freeBytes())+" free out of "+
            KIO::convertSize(temp.totalBytes())+" (";
  label.sprintf("%d",(100-temp.usedPerct()));
  label=templabel+label+"%) on [ "+temp.mntPoint()+" ("+temp.type()+") ]";
  status->setText(label);
}

QString ListPanel::parseSize(unsigned long val){
  QString temp,size;
  temp.sprintf("%lu",val);
  if(temp.length() <=3 ) return temp+"  ";
  unsigned int i = temp.length()%3;
  if (i==0) i=3;
  size = temp.left(i)+",";
  while(i+3 < temp.length()){
    size = size + temp.mid(i,3)+",";
    i+=3;
  }
  size = size + temp.right(3);
  return size+"  ";
}

QString ListPanel::shortPath(){
  int i=QFontMetrics(origin->font()).width("W");
	int j=QFontMetrics(origin->font()).width("0");
	j=(i>j ? i : j);
  unsigned int size = width()/j;

  QString s = virtualPath;
  if(size > s.length()) return s;
  s = s.right(size-2);
  s.remove(0,s.find('/'));
  return ".."+s.right(size-2);
}

// calculate item under the mouse, keep it and rebuild the old one
QListViewItem* ListPanel::itemOn(QPoint p) {
  // if there is a previous dealt-with item, rebuild it's pixmap
  if (currDragItem) currDragItem->setPixmap(0,currDragPix);
  // find the size of the listview's header
  int headerSize=height()-totals->height()-origin->height()-status->height()-fileList->visibleHeight();
  headerSize-=(fileList->horizontalScrollBar()->isVisible() ?
              fileList->horizontalScrollBar()->height() : 0);
//  QPoint evPos=mapToGlobal(p);
  p.setY(p.y()-headerSize);
  QListViewItem *i=fileList->itemAt(fileList->mapFromGlobal(p));
  if (!i) return 0;
  currDragItem=i;             // keep the current item's id and pixmap
  currDragPix=*i->pixmap(0);
  return i;
}

void ListPanel::dragEnterEvent ( QDragEnterEvent *ev ) {
  currDragItem=0; // just making sure
}

void ListPanel::dragLeaveEvent ( QDragLeaveEvent *ev ) {
  // if leaving panel, make sure we cleaned-up first.
  if (currDragItem) currDragItem->setPixmap(0,currDragPix);
  currDragItem=0;
}

void ListPanel::dragMoveEvent( QDragMoveEvent *ev ) {
  QStrList list;
  if (!QUriDrag::canDecode(ev) || !ev->provides("text/uri-list") ||
      !files->isWritable || !files->supportCopyTo ){
    ev->ignore(); // not for us to handle!
    return;
  } // if we got here, let's check the mouse location
  QPoint evPos=mapToGlobal(ev->pos());
  // calculate true width of fileList
  int hdelta=(fileList->horizontalScrollBar()->isVisible() ?
              fileList->horizontalScrollBar()->height() : 0);
  // create locations
  QPoint bottomRight(fileList->visibleWidth(),height()-totals->height()-hdelta);
  QPoint topLeft(0,bottomRight.y()-fileList->visibleHeight());
  topLeft=mapToGlobal(topLeft);
  bottomRight=mapToGlobal(bottomRight);
  if (evPos.y()<topLeft.y() || evPos.y()>bottomRight.y() ||
      evPos.x()<topLeft.x() || evPos.x()>bottomRight.x() ) {
     ev->ignore();  // if we're in a panel but out of the file list
     return;
  }
  QListViewItem *i=itemOn(evPos);
  // if we're pointing on an empty spot, leave
  if (!i) { ev->acceptAction(); return; }
  // otherwise, check if we're dragging on a directory
  if (i->text(1)=="<DIR>" && i->text(0)!="..") {
    i->setPixmap(0,folder_open_xpm);
  }
  ev->acceptAction();
}

void ListPanel::dropEvent( QDropEvent *ev ) {
  // if copyToPanel is true, then we call a simple vfs_addfiles
  bool copyToDirInPanel=false;
//  bool copyToZip=false;
  bool dragFromOtherPanel=false;
  bool dragFromThisPanel=false;
  bool isWritable=files->isWritable;

  bool MOVE=false;
  bool LINK=false;
  vfs* tempFiles = files;
  vfile *file;
  QPoint evPos=mapToGlobal(ev->pos());
  QListViewItem *i=itemOn(evPos);

  if (ev->source()==otherPanel) dragFromOtherPanel=true;
  if (ev->source()==this)       dragFromThisPanel=true;

  if (i){
     file=files->vfs_search(i->text(0));

     if (!file) { // trying to drop on the ".."
       if(virtualPath.right(1)=="\\") // root of archive..
          tempFiles=vfsStack.top();
       else copyToDirInPanel=true;
       isWritable=tempFiles->isWritable;
     }else{
      if (file->vfile_isDir()){
        copyToDirInPanel=true;
        isWritable = file->vfile_isWriteable();
        if (isWritable) {
          // keep the folder_open icon until we're finished, do it only
          // if the folder is writeable, to avoid flicker
          currDragPix=*i->pixmap(0);
          i->setPixmap(0,folder_open_xpm);
        }
      }else
        if (ev->source()==this) return ;// no dragging onto ourselves
    }
  } else    // if dragged from this panel onto an empty spot in the panel...
  if (dragFromThisPanel) {  // leave!
    ev->ignore();
    return;
  }

  if(!isWritable){
    ev->ignore();
    return;
  }
  //////////////////////////////////////////////////////////////////////////////
  // decode the data
  KURL::List URLs;
  QStrList list;
  if (!QUriDrag::decode(ev,list)) {
    ev->ignore(); // not for us to handle!
    if (copyToDirInPanel) i->setPixmap(0,currDragPix); // clean-up
    return;
  } // now, the list of URLs is stored in 'list', we'll create a KURL::List
  QStrListIterator it(list);
  while (it) {
    URLs.append(*it);
    ++it;
  }
  // the KURL::List is finished, let's go
  // --> display the COPY/MOVE/LINK menu
  if (dragFromOtherPanel && ev->action()==QDropEvent::Move) MOVE=true; else {
    QPopupMenu popup;
    popup.insertItem(i18n("Copy Here"),1);
    if(otherPanel->files->supportMoveFrom && files->supportMoveTo)
      popup.insertItem(i18n("Move Here"),2);
    if (files->vfs_getType()=="normal" && otherPanel->files->vfs_getType()=="normal")
      popup.insertItem(i18n("Link Here"),3);
    popup.insertItem(i18n("Cancel"),4);
    int result=popup.exec(evPos);
    switch (result) {
      case 1 :
        MOVE=false;
        break;
      case 2 :
        MOVE=true;
        break;
      case 3 :
        LINK=true;
        break;
      case -1 :  // user pressed outside the menu
      case 4:
        if (copyToDirInPanel) i->setPixmap(0,currDragPix); // clean-up
        return; // cancel was pressed;
    }
  } // big bunch o' stuff is finished, lets go on...
  if (LINK) {

    if (copyToDirInPanel) i->setPixmap(0,currDragPix); // clean-up
    return;     // link stuff goes here
  }
  // if we got here, we either copy of move
  QString dir = "";
  if (copyToDirInPanel) {
    dir = i->text(0);
    i->setPixmap(0,currDragPix); // clean-up
  }
  QWidget *notify = (!ev->source() ? 0 : ev->source());
  tempFiles->vfs_addFiles(&URLs,MOVE,notify,dir);
}

void ListPanel::startDragging(int mode) {
	QStrList URIs;
	bool draggingSingle=false;
	QListViewItem *iterator=fileList->firstChild();
	while (iterator!=0) {
		if (iterator->isSelected() && iterator->text(0)!=QString("..")) {
			URIs.append( QUriDrag::localFileToUri(files->vfs_getFile(iterator->text(0))) );
		}
		iterator=iterator->itemBelow();
	}
	// if the list id empty, don't drag nothing
	switch (URIs.count()) {
	  case 0 : emit finishedDragging(); // if dragged an empty space, don't do it!
             return;
    case 1 : draggingSingle=true;     // if drag a single file, change icon
             break;
  }
  QUriDrag *d=new QUriDrag(URIs,this);
  if (draggingSingle) d->setPixmap(*fileList->currentItem()->pixmap(0),QPoint(-7,0));
                 else d->setPixmap(multi_xpm,QPoint(-7,0));
  emit finishedDragging();
  switch (mode) {
    case 0 : d->dragCopy(); break; // copy will be used for copy and for link
    case 1 : d->dragMove(); break;
  }
}

// pops a right-click menu for items
void ListPanel::popRightClickMenu(QListViewItem* item,QPoint loc) {
  // these are the values that will always exist in the menu
  #define OPEN_ID       90
  #define OPEN_WITH_ID  91
  #define OPEN_KONQ_ID  92
  #define OPEN_TERM_ID  93
  #define CHOOSE_ID     94
  #define DELETE_ID     95
  #define COPY_ID       96
  #define MOVE_ID       97
  #define RENAME_ID     98
  #define PROPERTIES_ID 99
  //////////////////////////////////////////////////////////
  if (item->text(0)=="..") return;
  vfile *file=file=files->vfs_search(item->text(0));
  QString localFile=files->vfs_getFile(item->text(0));
  KURL u;
  u.setPath(localFile);
  KURL::List lst;
  lst.append(u);
  // create the menu
  QPopupMenu popup,openWith;
  // the OPEN option - generic "execute"
  popup.insertItem("Open/Run",OPEN_ID);      // create the open option
  popup.changeItem(OPEN_ID,*item->pixmap(0), // and add pixmap
     i18n((file->vfile_isExecutable())&&(!file->vfile_isDir()) ? "Run" : "Open"));
  popup.insertSeparator();
  // Open with
  // try to find-out which apps can open the file
//KURL url;
//url.setPath(files->vfs_getFile(item->text(0)));
//KFileItem fi(-1,-1,url);
//KTrader *trader = KdedInstance::self()->ktrader();
//KTrader::OfferList offers = trader->query(fi.mimetype());
//if (offers.count()>0) cerr << ">0";
  openWith.insertItem(i18n("Konqueror"),OPEN_KONQ_ID);
  openWith.insertItem("Terminal",OPEN_TERM_ID);
  openWith.changeItem(OPEN_TERM_ID,QPixmap(terminal_xpm),i18n("Terminal"));
  openWith.insertSeparator();
  openWith.insertItem(i18n("Other..."),CHOOSE_ID);
  popup.insertItem(QPixmap(),&openWith,OPEN_WITH_ID);
  popup.changeItem(OPEN_WITH_ID,"Open with");
  popup.insertSeparator();
  // COPY
  popup.insertItem(i18n("Copy"),COPY_ID);
  if (files->isWritable) {
    // MOVE
    popup.insertItem(i18n("Move"),MOVE_ID);
    // RENAME
    popup.insertItem(i18n("Rename"),RENAME_ID);
    // DELETE
    popup.insertItem(i18n("Delete"),DELETE_ID);
  }
  // PROPERTIES
  popup.insertSeparator();
  popup.insertItem(i18n("Properties"),PROPERTIES_ID);
  // run it, on the mouse location
  int i=QFontMetrics(popup.font()).height()*2;
  int result=popup.exec(QPoint(loc.x()+5,loc.y()+i));
  // check out the user's option
  switch (result) {
    case -1 : return;     // the user clicked outside of the menu
    case OPEN_ID :        // Open/Run
      slotExecuted(item);
      break;
    case COPY_ID :
      func->copyFiles();
      break;
    case MOVE_ID :
      func->moveFiles();
      break;
    case RENAME_ID :
      func->rename();
      break;
    case DELETE_ID :
      func->deleteFiles();
      break;
    case PROPERTIES_ID :  // Properties
      slotFileProperties(item);
      break;
    case OPEN_KONQ_ID :   // open in konqueror
      kapp->startServiceByDesktopName("konqueror",localFile);
      break;
    case CHOOSE_ID :      // Other...
      new KFileOpenWithHandler();
      KOpenWithHandler::getOpenWithHandler()->displayOpenWithDialog(lst);
      break;
    case OPEN_TERM_ID :   // open in terminal
		  QString save = getcwd(0,0);
		  KShellProcess proc;
		  chdir( (file->vfile_isDir() ? localFile : realPath) );
      krConfig->setGroup("External");
		  proc << krConfig->readEntry("Terminal",_Terminal);
      if (!file->vfile_isDir()) proc << krConfig->readEntry("Terminal Exec",_TerminalExec) << item->text(0);
  	  proc.start(KProcess::DontCare);
		  chdir(save);
      break;
  }
}
