/***************************************************************************   
                       normal_vfs.cpp                             
                   -------------------
    Version							 : Milestone 1
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Rafi Yanai
 ---------------------------------------------------------------------------

 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// KDE includes
#include <kmimetype.h>
#include <kio/jobclasses.h>
#include <klocale.h>
// Krusader includes
#include "normal_vfs.h"
#include "krdialogs.h"
#include "krusader.h"
// QT includes
#include <qdir.h>
#include <qmessagebox.h>

normal_vfs::normal_vfs(QString origin,QWidget* panel):vfs(panel),t(0){
  supportCopyTo = true;
  supportMoveFrom = true;     // the big 4 is supported by normal vfs...
 	supportDelete = true;
 	supportMoveTo = true;
 	
  vfs_filesP = &vfs_files;
  vfs_files.setAutoDelete(true);
	
	vfs_type="normal";
	
	// connect the watcher to vfs_refresh
	connect(&watcher,SIGNAL(dirty()),this,SLOT(vfs_refresh()));
}

bool normal_vfs::vfs_refresh(const QString& origin){
	if(!active) return true;
	
	krMtMan.autoMount(origin);
	
	watcher.stopScan(); //stop watching the old dir
	watcher.clearList();
	
	if(t != 0 ) delete t;	
	
	// check that the new origin exists
	QFileInfo temp(origin);
	if( !temp.exists() || !temp.isDir() ) return false;
	
	// set the writable attribute
	struct stat stat_p;
  stat(origin,&stat_p);
  mode_t m = stat_p.st_mode;
  if (geteuid() == 0) isWritable = true;
  else if( m&0002 ) isWritable = true;
  else if( temp.groupId()==getegid() && m&0020 ) isWritable = true;
  else if( temp.ownerId()==geteuid() && m&0200 ) isWritable = true;
  else isWritable = false;

 	dir.setPath(origin);
	
	krConfig->setGroup("General");
	//////////////////////////////
	mtm=krConfig->readBoolEntry("Mimetype Magic",_MimetypeMagic);
	int hidden = 0;
	if (krConfig->readBoolEntry("Show Hidden",_ShowHidden)) hidden = (QDir::Hidden);
	dir.setFilter(QDir::All|hidden);
	dir.setSorting(QDir::Name|QDir::DirsFirst);
	
	// set the origin...
	vfs_origin = dir.path();
  // clear the the list
	vfs_files.clear();
	
	// get the first file
	QFileInfoList* fileList = const_cast<QFileInfoList*>(dir.entryInfoList());
	qfi = fileList->first();
	// as long as u can find files - add them to the vfs

  emit startUpdate();

  t = new QTimer(this);
  connect( t,SIGNAL(timeout()),SLOT(updateList()));
  t->start(0,FALSE);

  return true;
}

void normal_vfs::updateList(){
  if(qfi == 0){
    if (t != 0 ) delete t;
    t = 0;
    emit endUpdate();
    watcher.addDir(dir.path()); //start watching the new dir
    watcher.startScan();
    return;
  }
  QFileInfoList* fileList = const_cast<QFileInfoList*>(dir.entryInfoList());

  // add 25 files
  for(int i =0; i<25 && qfi != 0 ;++i){
    // get file statistics
	  QString name=qfi->fileName();
	  if (name=="." || name == ".."){ // we dont need the ".",".." enteries
	    qfi = fileList->next();
	    continue;
	  }		
	
	  struct stat stat_p;
	  lstat(vfs_origin+"/"+name,&stat_p);
	  unsigned long size = stat_p.st_size;
    QString perm = mode2QString(stat_p.st_mode);
    if(qfi->isDir()) perm[0] = 'd';
	  QString	dateTime=time2QString(stat_p.st_mtime);
	  bool symLink=qfi->isSymLink();
	  QString owner=qfi->owner();
	  QString group=qfi->group();
	  QString mime=KMimeType::findByURL( qfi->filePath(),stat_p.st_mode,false,!mtm)->name();
	  QString symDest = ""; if(symLink)symDest= qfi->readLink();
	
	  // create a new virtual file object
    vfile* temp=new vfile(name,size,perm,dateTime,symLink,owner,group,mime,symDest,stat_p.st_mode);
    vfs_addToList(temp);
	  qfi = fileList->next();
  }
}

// copy a file to the vfs (physical)	
void normal_vfs::vfs_addFiles(KURL::List *fileUrls,bool move,QWidget* toNotify,QString dir){
  watcher.stopScan(); // we will refresh manually this time...	

	KIO::Job* job = new KIO::CopyJob(*fileUrls,vfs_origin+"/"+dir,move,false,false );
  new KRprogress(job);
  connect(job,SIGNAL(result(KIO::Job*)),this,SLOT(vfs_refresh()) );
  if(move) // notify the other panel
    connect(job,SIGNAL(result(KIO::Job*)),toNotify,SLOT(slotPanelRefresh()) );
}

// remove a file from the vfs (physical)
void normal_vfs::vfs_delFiles(QValueList<QString> *fileNames){
	KURL::List filesUrls;
	KURL url;
	QDir local(vfs_origin);
  vfile* vf;

  watcher.stopScan(); // we will refresh manually this time...	

	// names -> urls
	for(uint i=0 ; i<fileNames->count(); ++i){
		QString filename = (*fileNames)[i];
		vf = vfs_search(filename);
		url.setPath( vfs_origin+"/"+filename);
		filesUrls.append(url);
	}
	KIO::Job *job = new KIO::DeleteJob(filesUrls, false, false);
	new KRprogress(job);
	connect(job,SIGNAL(result(KIO::Job*)),this,SLOT(vfs_refresh()));
}

// return a path to the file
QString normal_vfs::vfs_getFile(QString name){	
	if (vfs_search(name)->vfile_isDir())
		return vfs_origin+"/"+name+"/";
	return vfs_origin+"/"+name;
}

void normal_vfs::vfs_mkdir(QString name){
	if (!dir.mkdir(name)){
	  QMessageBox mb( i18n("Can't create directory - Krusader"),
		i18n("Can't create a directory\n check your permissions"),
		QMessageBox::Critical,
		QMessageBox::Ok | QMessageBox::Default| QMessageBox::Escape,
		0,0 );
    mb.exec();
	}	
	else vfs_refresh();
}


void normal_vfs::vfs_rename(QString fileName,QString newName){
  KURL::List fileUrls;
  KURL url;

  watcher.stopScan(); // we will refresh manually this time...	

  url.setPath( vfs_origin+"/"+fileName );
  fileUrls.append(url);

  KIO::Job *job = new KIO::CopyJob(fileUrls,vfs_origin+"/"+newName,true,false,false );
	connect(job,SIGNAL(result(KIO::Job*)),this,SLOT(vfs_refresh()));
}
