/***************************************************************************
                                treepanel.cpp
                             -------------------
    begin                : Thu May 4 2000
    copyright            : (C) 2000 by Shie Erlich & Rafi Yanai
 ---------------------------------------------------------------------------
  Description 
 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// KDE includes
#include <klocale.h>
// Krusader includes
#include "kvfspanel.h"
#include "krlistitem.h"
#include "krusader.h"
#include "defaults.h"

////////////////////////////////////////////////////
//           The TREE panel constructor           //
////////////////////////////////////////////////////
TreePanel::TreePanel(QWidget *parent,const char *name) : KVFSPanel(parent,name) {

  type = "tree";
	func = new TreePanelFunc(this);
	setAcceptDrops(false);
	layout=new QGridLayout(this,4,1);
  // totals will mark how many dirs are marked
  totals = new QLabel(this);
  krConfig->setGroup("Look&Feel");
  totals->setFont(krConfig->readFontEntry("Bookmark Font",_BookmarkFont));
  totals->setFrameStyle( QFrame::Box | QFrame::Raised);
  totals->setLineWidth(1);		// a nice 3D touch :-)

 	origin = new QPushButton("Tree View",this);
  krConfig->setGroup("Look&Feel");
	origin->setFont(krConfig->readFontEntry("Filelist Font",_FilelistFont));
	origin->font().setBold(true);
	origin->font().setPointSize(origin->font().pointSize()+2);		
		// pressing the origin bar sets focus
	connect(origin,SIGNAL(clicked()),this,SLOT(slotFocusOnMe()));

	status = new QLabel(this);
  status->setFont(krConfig->readFontEntry("Bookmark Font",_BookmarkFont));
  status->setFrameStyle( QFrame::Box | QFrame::Raised);
  status->setText("");        // needed for initialization code!

  status->setLineWidth(1);		// a nice 3D touch :-)
  fileList = new KFileList(this); // the same KFilelist can do trees!
  fileList->setFont(krConfig->readFontEntry("Filelist Font",_FilelistFont));
	fileList->setMultiSelection(false);
	fileList->setVScrollBarMode(QScrollView::AlwaysOn);
	fileList->setHScrollBarMode(QScrollView::Auto);
	fileList->setRootIsDecorated(true);
	fileList->setShowSortIndicator(true);	
	int i=QFontMetrics(fileList->font()).width("W");
	fileList->addColumn("Name",i*37); // create an even look between the panels
	// connections
	connect(fileList, SIGNAL(expanded(QListViewItem*)),
					this,SLOT(slotExpandTree(QListViewItem*)));
	connect(fileList, SIGNAL(collapsed(QListViewItem*)),
					this,SLOT(slotCollapsedTree(QListViewItem*)));
  	// double click and pressing return executes
  connect(fileList, SIGNAL(doubleClicked(QListViewItem *)), this,
          SLOT(slotChangeFromTree(QListViewItem*)));
  connect(fileList, SIGNAL(returnPressed(QListViewItem *)), this,
          SLOT(slotChangeFromTree(QListViewItem*)));
		// even a "drag-swush" through the list changes focus
	connect(fileList, SIGNAL(mouseButtonPressed(int,QListViewItem *,const QPoint &,int)),
					this,SLOT(slotFocusOnMe()));
	  // and not forgetting to reroute active panel!!
	connect(this,SIGNAL(activePanelChanged(KVFSPanel *)),this->parent()->parent(),
					SLOT(slotSetActivePanel(KVFSPanel *)) );		

	// finish the layout
	layout->addWidget(origin,0,0);
	layout->addWidget(status,1,0);
	layout->addWidget(fileList,2,0);
  layout->addWidget(totals,3,0);
}

// this is used to start the panel, AFTER setOther() has been used
//////////////////////////////////////////////////////////////////
void TreePanel::start(bool left) {
	slotRecreateTree("/");
}

void TreePanel::slotRecreateTree(QString path) {
  QListViewItem *item;

  fileList->clear();
  // insert the root item to the list manually
  item=new QListViewItem(fileList,"/");
  int i=readTree("/",item);
  if (i>0) {
    item->setExpandable(true); // root has children ...
    item->setOpen(true);       // ... show them
    item->setPixmap(0,folder_open_xpm);
  } else item->setPixmap(0,folder_xpm);
  fileList->triggerUpdate();
}

// readTree reads the tree and creates a tree-view Panel
/////////////////////////////////////////////////////
int TreePanel::readTree(QString path,QListViewItem *father) {
  KRListItem *item;
  int i=0;
  QDir d(path);

  // read only true directories (no symlinks), including hidden and sort by name
  d.setFilter( QDir::Dirs | QDir::Hidden | QDir::NoSymLinks );
  d.setSorting(QDir::Name);

  const QFileInfoList *list = d.entryInfoList();
  QFileInfoListIterator it( *list );      // create list iterator
  QFileInfo *fi;                          // pointer for traversing

  while ( (fi=it.current()) ) {           // for each directory...
    if ((fi->fileName()!=".") && (fi->fileName()!="..")) {
      item=new KRListItem(father,fi->fileName());
      item->setPixmap(0,folder_xpm);
      // this part checks if the dir should have "expandable" icon
      ////////////////////////////////////////////////////////////
      QDir temp(path+"/"+fi->fileName());
      temp.setFilter( QDir::Dirs | QDir::Hidden | QDir::NoSymLinks );
      if (temp.count()>2) item->setExpandable(true);
      ////////////////////////////////////////////////////////////
      i++;                                // counter
    }
    ++it;                               // goto next list element
  }
  return i;
}

void TreePanel::slotExpandTree(QListViewItem *item) {
  // if the item has children already, no need to re-read
  if (!item->firstChild()) {
    // otherwise, it's the first time we're expanding it
    QString path=returnPath(item);
    // now, call readTree to do it's job
    readTree(path,item);
  }
  item->setPixmap(0,folder_open_xpm); // set the icon
  fileList->triggerUpdate();          // and update the list
}

void TreePanel::slotCollapsedTree(QListViewItem *item) {
  item->setPixmap(0,folder_xpm); // set the icon
  fileList->triggerUpdate();          // and update the list
}

// returns a full path originating at root until hitting the ITEM
/////////////////////////////////////////////////////////////////
QString TreePanel::returnPath(QListViewItem *item) {
  QString path=item->text(0);
  QString s;
  // 1st, create the full path to the expanded directory
  QListViewItem *iterator=item->parent();
  while (iterator!=0) {
    s=iterator->text(0)=="/" ? "" : "/";
    path=(iterator->text(0))+s+path;
    iterator=iterator->parent();
  }
  return path;
}
