/***************************************************************************   
                       		vfile.cpp
                   		-------------------
    Version							 : Milestone 1
    begin                : Sun May 7 2000
    copyright            : (C) 2000 by Rafi Yanai
 ---------------------------------------------------------------------------

 ***************************************************************************

  A 

     db   dD d8888b. db    db .d8888.  .d8b.  d8888b. d88888b d8888b.
     88 ,8P' 88  `8D 88    88 88'  YP d8' `8b 88  `8D 88'     88  `8D
     88,8P   88oobY' 88    88 `8bo.   88ooo88 88   88 88ooooo 88oobY'
     88`8b   88`8b   88    88   `Y8b. 88~~~88 88   88 88~~~~~ 88`8b
     88 `88. 88 `88. 88b  d88 db   8D 88   88 88  .8D 88.     88 `88.
     YP   YD 88   YD ~Y8888P' `8888Y' YP   YP Y8888D' Y88888P 88   YD

                                                     S o u r c e    F i l e

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// Krusader includes
#include "vfile.h"
//System includes
#include <pwd.h>
#include <stdio.h>
#include <grp.h>


vfile::IntCache   * vfile::passwdCache = 0L;
vfile::IntCache   * vfile::groupCache  = 0L;

vfile::vfile(QString name,	                  // useful construtor
						unsigned long size,	
						QString perm,
						QString	dateTime,
						bool symLink,
						uid_t	owner,
						gid_t group,
						QString mime,
						QString symDest,
						mode_t mode){
	vfile_name=name;
	vfile_size=size;
	vfile_owner=owner;
	vfile_group=group;
	vfile_perm=perm;
	vfile_dateTime=dateTime;
	vfile_symLink=symLink;
	vfile_mimeType=mime;
	vfile_symDest=symDest;
	vfile_mode=mode;
  if (!passwdCache) initCaches();
	vfile_owner=QString::fromLocal8Bit(passwdCache->find(owner));
	vfile_group=QString::fromLocal8Bit(groupCache->find(group));
}

// init the passwd-group cahes
void vfile::initCaches(){	
	passwdCache=new IntCache(317);
	groupCache=new IntCache(317);
					
	passwdCache->setAutoDelete(true);
	groupCache->setAutoDelete(true);
	
	struct passwd *pass;
  while ( (pass = getpwent()) != 0L ) {
	  passwdCache->insert(pass->pw_uid, qstrdup(pass->pw_name));
  }
  delete pass;
  endpwent();

  struct group *gr;
  while( (gr = getgrent()) != 0L ) {
	  groupCache->insert(gr->gr_gid, qstrdup(gr->gr_name));
  }
	delete gr;
  endgrent();
}

vfile::vfile(QString name,	                  // useful construtor
						unsigned long size,	
						QString perm,
						QString	dateTime,
						bool symLink,
						QString	owner,
						QString group,
						QString mime,
						QString symDest,
						mode_t mode){
		vfile_name=name;
		vfile_size=size;
		vfile_owner=owner;
		vfile_group=group;
		vfile_perm=perm;
		vfile_dateTime=dateTime;
		vfile_symLink=symLink;
		vfile_mimeType=mime;
		vfile_symDest=symDest;
		vfile_mode=mode;
		if (!passwdCache) initCaches();
}
	


bool vfile::vfile_isReadable(){
  // first check other permissions.
	if(vfile_perm[7]!='-') return true;
	// now check group permission
	if((vfile_perm[4]!='-')&&
	  (vfile_group==QString::fromLocal8Bit(groupCache->find(getgid()) )))
		return true;
	// the last chance - user permissions
	if((vfile_perm[1]!='-')&&
	  (vfile_owner==QString::fromLocal8Bit(passwdCache->find(getuid()) )))
		return true;
	// sorry !
	return false;
}

bool vfile::vfile_isWriteable(){
  // first check other permissions.
	if(vfile_perm[8]!='-') return true;
	// now check group permission
	if((vfile_perm[5]!='-')&&
	  (vfile_group==QString::fromLocal8Bit(groupCache->find(getegid()) )))
		return true;
	// the last chance - user permissions
	if((vfile_perm[2]!='-')&&
	  (vfile_owner==QString::fromLocal8Bit(passwdCache->find(geteuid()) )))
		return true;
	// sorry !
	return false;
}

bool vfile::vfile_isExecutable(){
  // first check other permissions.
	if(vfile_perm[9]!='-') return true;
	// now check group permission
	if((vfile_perm[6]!='-')&&
	  (vfile_group==QString::fromLocal8Bit(groupCache->find(getegid()) )))
		return true;
	// the last chance - user permissions
	if((vfile_perm[3]!='-')&&
	  (vfile_owner==QString::fromLocal8Bit(passwdCache->find(geteuid()) )))
		return true;
	// sorry !
	return false;
}
